// CubeSearch v1.0
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains all of the code for simulating a Rubik's Cube and finding a solution a Cube.

#region Using statements

using System;
using System.Collections;
using BenTools.Data;
using System.IO;
using System.Threading;

#endregion

namespace Cube_Search {

	public class Cube	{

		#region Variables and initialization code

		public int[,] iFace;                   // The data for each color at each position on each face (i.e., the cube configuration)
		public int iFrontFaceView, iFrontFaceRotation;   // Specifies the current view of the cube (which face, and rotated how, is in fring?)
		public int[,,,] iRotateFaceView;  // given a face, a rotation, an axis, and a direction, which face is now in front?
		public int[,,,] iRotateFaceRotation;  // given a face, a rotation, an axis, and a direction, how is the front face currently rotated?
		public Transform[,,,,] tTransform;  // given a cube state, an axis, and a direction, perform the specified transform on the cube
		public int[,] iTopFaceView;  // given a front face number and rotation, which face is on top?
		public int[,] iTopFaceRotation;  // given a front face number and rotation, how is the top face rotated?
		public int[,] iRightFaceView;  // given a host front face number and rotation, which face is on the right?
		public int[,] iRightFaceRotation;  // given a front face number and rotation, how is the right face rotated?
		public ArrayList aSolution = null;  // the array of steps to a solution
		public string strStatus;  // the status of the cube (updated during the solution process)
		public SolutionStep solution = null;  // the container for a state in the thread solution list
		public Thread thread = null;     // the solution-finder thread (NULL if none)
		public bool bStop = false;  // a Boolean indicating that the cube should stop processing commands
		public bool bDone = false;  // a Boolean indicating that the cube has been solved and a decision posted to the solution list
		public int iThreshold = 100;  // the threshold needed to activate the security features (to prevent lock-outs)

		public Cube() {  // constructor
			// setup manipulation reference arrays
			// Variables and initialization code
			iRotateFaceView = new int[6, 4, 3, 6];  // given a face, a rotation, an axis,  and a direction, what face is now in front?
			iRotateFaceRotation = new int[6, 4, 3, 6];  // given a face, a rotation, an axis, and a direction, how is the front face now rotated?
			// hard-code the data for viewing the cube from different angles
			SetRotateFaceView(1, 1, Axis.X, Direction.Left, 2, 1);
			SetRotateFaceView(1, 1, Axis.X, Direction.Right, 4, 1);
			SetRotateFaceView(1, 1, Axis.Y, Direction.Down, 5, 1);
			SetRotateFaceView(1, 1, Axis.Y, Direction.Up, 6, 1);
			SetRotateFaceView(1, 1, Axis.Z, Direction.CCW, 1, 2);
			SetRotateFaceView(1, 1, Axis.Z, Direction.CW, 1, 4);
			SetRotateFaceView(1, 2, Axis.X, Direction.Left, 6, 2);
			SetRotateFaceView(1, 2, Axis.X, Direction.Right, 5, 2);
			SetRotateFaceView(1, 2, Axis.Y, Direction.Down, 2, 2);
			SetRotateFaceView(1, 2, Axis.Y, Direction.Up, 4, 2);
			SetRotateFaceView(1, 2, Axis.Z, Direction.CCW, 1, 3);
			SetRotateFaceView(1, 2, Axis.Z, Direction.CW, 1, 1);
			SetRotateFaceView(1, 3, Axis.X, Direction.Left, 4, 3);
			SetRotateFaceView(1, 3, Axis.X, Direction.Right, 2, 3);
			SetRotateFaceView(1, 3, Axis.Y, Direction.Down, 6, 3);
			SetRotateFaceView(1, 3, Axis.Y, Direction.Up, 5, 3);
			SetRotateFaceView(1, 3, Axis.Z, Direction.CCW, 1, 4);
			SetRotateFaceView(1, 3, Axis.Z, Direction.CW, 1, 2);
			SetRotateFaceView(1, 4, Axis.X, Direction.Left, 5, 4);
			SetRotateFaceView(1, 4, Axis.X, Direction.Right, 6, 4);
			SetRotateFaceView(1, 4, Axis.Y, Direction.Down, 4, 4);
			SetRotateFaceView(1, 4, Axis.Y, Direction.Up, 2, 4);
			SetRotateFaceView(1, 4, Axis.Z, Direction.CCW, 1, 1);
			SetRotateFaceView(1, 4, Axis.Z, Direction.CW, 1, 3);
			SetRotateFaceView(2, 1, Axis.X, Direction.Left, 3, 1);
			SetRotateFaceView(2, 1, Axis.X, Direction.Right, 1, 1);
			SetRotateFaceView(2, 1, Axis.Y, Direction.Down, 5, 4);
			SetRotateFaceView(2, 1, Axis.Y, Direction.Up, 6, 2);
			SetRotateFaceView(2, 1, Axis.Z, Direction.CCW, 2, 2);
			SetRotateFaceView(2, 1, Axis.Z, Direction.CW, 2, 4);
			SetRotateFaceView(2, 2, Axis.X, Direction.Left, 6, 3);
			SetRotateFaceView(2, 2, Axis.X, Direction.Right, 5, 1);
			SetRotateFaceView(2, 2, Axis.Y, Direction.Down, 3, 2);
			SetRotateFaceView(2, 2, Axis.Y, Direction.Up, 1, 2);
			SetRotateFaceView(2, 2, Axis.Z, Direction.CCW, 2, 3);
			SetRotateFaceView(2, 2, Axis.Z, Direction.CW, 2, 1);
			SetRotateFaceView(2, 3, Axis.X, Direction.Left, 1, 3);
			SetRotateFaceView(2, 3, Axis.X, Direction.Right, 3, 3);
			SetRotateFaceView(2, 3, Axis.Y, Direction.Down, 6, 4);
			SetRotateFaceView(2, 3, Axis.Y, Direction.Up, 5, 2);
			SetRotateFaceView(2, 3, Axis.Z, Direction.CCW, 2, 4);
			SetRotateFaceView(2, 3, Axis.Z, Direction.CW, 2, 2);
			SetRotateFaceView(2, 4, Axis.X, Direction.Left, 5, 3);
			SetRotateFaceView(2, 4, Axis.X, Direction.Right, 6, 1);
			SetRotateFaceView(2, 4, Axis.Y, Direction.Down, 1, 4);
			SetRotateFaceView(2, 4, Axis.Y, Direction.Up, 3, 4);
			SetRotateFaceView(2, 4, Axis.Z, Direction.CCW, 2, 1);
			SetRotateFaceView(2, 4, Axis.Z, Direction.CW, 2, 3);
			SetRotateFaceView(3, 1, Axis.X, Direction.Left, 4, 1);
			SetRotateFaceView(3, 1, Axis.X, Direction.Right, 2, 1);
			SetRotateFaceView(3, 1, Axis.Y, Direction.Down, 5, 3);
			SetRotateFaceView(3, 1, Axis.Y, Direction.Up, 6, 3);
			SetRotateFaceView(3, 1, Axis.Z, Direction.CCW, 3, 2);
			SetRotateFaceView(3, 1, Axis.Z, Direction.CW, 3, 4);
			SetRotateFaceView(3, 2, Axis.X, Direction.Left, 6, 4);
			SetRotateFaceView(3, 2, Axis.X, Direction.Right, 5, 4);
			SetRotateFaceView(3, 2, Axis.Y, Direction.Down, 4, 2);
			SetRotateFaceView(3, 2, Axis.Y, Direction.Up, 2, 2);
			SetRotateFaceView(3, 2, Axis.Z, Direction.CCW, 3, 3);
			SetRotateFaceView(3, 2, Axis.Z, Direction.CW, 3, 1);
			SetRotateFaceView(3, 3, Axis.X, Direction.Left, 2, 3);
			SetRotateFaceView(3, 3, Axis.X, Direction.Right, 4, 3);
			SetRotateFaceView(3, 3, Axis.Y, Direction.Down, 6, 1);
			SetRotateFaceView(3, 3, Axis.Y, Direction.Up, 5, 1);
			SetRotateFaceView(3, 3, Axis.Z, Direction.CCW, 3, 4);
			SetRotateFaceView(3, 3, Axis.Z, Direction.CW, 3, 2);
			SetRotateFaceView(3, 4, Axis.X, Direction.Left, 5, 2);
			SetRotateFaceView(3, 4, Axis.X, Direction.Right, 6, 2);
			SetRotateFaceView(3, 4, Axis.Y, Direction.Down, 2, 4);
			SetRotateFaceView(3, 4, Axis.Y, Direction.Up, 4, 4);
			SetRotateFaceView(3, 4, Axis.Z, Direction.CCW, 3, 1);
			SetRotateFaceView(3, 4, Axis.Z, Direction.CW, 3, 3);
			SetRotateFaceView(4, 1, Axis.X, Direction.Left, 1, 1);
			SetRotateFaceView(4, 1, Axis.X, Direction.Right, 3, 1);
			SetRotateFaceView(4, 1, Axis.Y, Direction.Down, 5, 2);
			SetRotateFaceView(4, 1, Axis.Y, Direction.Up, 6, 4);
			SetRotateFaceView(4, 1, Axis.Z, Direction.CCW, 4, 2);
			SetRotateFaceView(4, 1, Axis.Z, Direction.CW, 4, 4);
			SetRotateFaceView(4, 2, Axis.X, Direction.Left, 6, 1);
			SetRotateFaceView(4, 2, Axis.X, Direction.Right, 5, 3);
			SetRotateFaceView(4, 2, Axis.Y, Direction.Down, 1, 2);
			SetRotateFaceView(4, 2, Axis.Y, Direction.Up, 3, 2);
			SetRotateFaceView(4, 2, Axis.Z, Direction.CCW, 4, 3);
			SetRotateFaceView(4, 2, Axis.Z, Direction.CW, 4, 1);
			SetRotateFaceView(4, 3, Axis.X, Direction.Left, 3, 3);
			SetRotateFaceView(4, 3, Axis.X, Direction.Right, 1, 3);
			SetRotateFaceView(4, 3, Axis.Y, Direction.Down, 6, 2);
			SetRotateFaceView(4, 3, Axis.Y, Direction.Up, 5, 4);
			SetRotateFaceView(4, 3, Axis.Z, Direction.CCW, 4, 4);
			SetRotateFaceView(4, 3, Axis.Z, Direction.CW, 4, 2);
			SetRotateFaceView(4, 4, Axis.X, Direction.Left, 5, 1);
			SetRotateFaceView(4, 4, Axis.X, Direction.Right, 6, 3);
			SetRotateFaceView(4, 4, Axis.Y, Direction.Down, 3, 4);
			SetRotateFaceView(4, 4, Axis.Y, Direction.Up, 1, 4);
			SetRotateFaceView(4, 4, Axis.Z, Direction.CCW, 4, 1);
			SetRotateFaceView(4, 4, Axis.Z, Direction.CW, 4, 3);
			SetRotateFaceView(5, 1, Axis.X, Direction.Left, 2, 2);
			SetRotateFaceView(5, 1, Axis.X, Direction.Right, 4, 4);
			SetRotateFaceView(5, 1, Axis.Y, Direction.Down, 3, 3);
			SetRotateFaceView(5, 1, Axis.Y, Direction.Up, 1, 1);
			SetRotateFaceView(5, 1, Axis.Z, Direction.CCW, 5, 2);
			SetRotateFaceView(5, 1, Axis.Z, Direction.CW, 5, 4);
			SetRotateFaceView(5, 2, Axis.X, Direction.Left, 1, 2);
			SetRotateFaceView(5, 2, Axis.X, Direction.Right, 3, 4);
			SetRotateFaceView(5, 2, Axis.Y, Direction.Down, 2, 3);
			SetRotateFaceView(5, 2, Axis.Y, Direction.Up, 4, 1);
			SetRotateFaceView(5, 2, Axis.Z, Direction.CCW, 5, 3);
			SetRotateFaceView(5, 2, Axis.Z, Direction.CW, 5, 1);
			SetRotateFaceView(5, 3, Axis.X, Direction.Left, 4, 2);
			SetRotateFaceView(5, 3, Axis.X, Direction.Right, 2, 4);
			SetRotateFaceView(5, 3, Axis.Y, Direction.Down, 1, 3);
			SetRotateFaceView(5, 3, Axis.Y, Direction.Up, 3, 1);
			SetRotateFaceView(5, 3, Axis.Z, Direction.CCW, 5, 4);
			SetRotateFaceView(5, 3, Axis.Z, Direction.CW, 5, 2);
			SetRotateFaceView(5, 4, Axis.X, Direction.Left, 3, 2);
			SetRotateFaceView(5, 4, Axis.X, Direction.Right, 1, 4);
			SetRotateFaceView(5, 4, Axis.Y, Direction.Down, 4, 3);
 			SetRotateFaceView(5, 4, Axis.Y, Direction.Up, 2, 1);
			SetRotateFaceView(5, 4, Axis.Z, Direction.CCW, 5, 1);
			SetRotateFaceView(5, 4, Axis.Z, Direction.CW, 5, 3);
			SetRotateFaceView(6, 1, Axis.X, Direction.Left, 2, 4);
			SetRotateFaceView(6, 1, Axis.X, Direction.Right, 4, 2);
			SetRotateFaceView(6, 1, Axis.Y, Direction.Down, 1, 1);
			SetRotateFaceView(6, 1, Axis.Y, Direction.Up, 3, 3);
			SetRotateFaceView(6, 1, Axis.Z, Direction.CCW, 6, 2);
			SetRotateFaceView(6, 1, Axis.Z, Direction.CW, 6, 4);
			SetRotateFaceView(6, 2, Axis.X, Direction.Left, 3, 4);
			SetRotateFaceView(6, 2, Axis.X, Direction.Right, 1, 2);
			SetRotateFaceView(6, 2, Axis.Y, Direction.Down, 2, 1);
			SetRotateFaceView(6, 2, Axis.Y, Direction.Up, 4, 3);
			SetRotateFaceView(6, 2, Axis.Z, Direction.CCW, 6, 3);
			SetRotateFaceView(6, 2, Axis.Z, Direction.CW, 6, 1);
			SetRotateFaceView(6, 3, Axis.X, Direction.Left, 4, 4);
			SetRotateFaceView(6, 3, Axis.X, Direction.Right, 2, 2);
			SetRotateFaceView(6, 3, Axis.Y, Direction.Down, 3, 1);
			SetRotateFaceView(6, 3, Axis.Y, Direction.Up, 1, 3);
			SetRotateFaceView(6, 3, Axis.Z, Direction.CCW, 6, 4);
			SetRotateFaceView(6, 3, Axis.Z, Direction.CW, 6, 2);
			SetRotateFaceView(6, 4, Axis.X, Direction.Left, 1, 4);
			SetRotateFaceView(6, 4, Axis.X, Direction.Right, 3, 2);
			SetRotateFaceView(6, 4, Axis.Y, Direction.Down, 4, 1);
			SetRotateFaceView(6, 4, Axis.Y, Direction.Up, 2, 3);
			SetRotateFaceView(6, 4, Axis.Z, Direction.CCW, 6, 1);
			SetRotateFaceView(6, 4, Axis.Z, Direction.CW, 6, 3);
			// create face view data
			iTopFaceView = new int[6,4];
			iTopFaceRotation = new int[6,4];
			iRightFaceView = new int[6,4];
			iRightFaceRotation = new int[6,4];
			SetFaceView(1, 1, 5, 1, 2, 1);
			SetFaceView(1, 2, 2, 2, 6, 2);
			SetFaceView(1, 3, 6, 3, 4, 3);
			SetFaceView(1, 4, 4, 4, 5, 4);
			SetFaceView(2, 1, 5, 4, 3, 1);
			SetFaceView(2, 2, 3, 2, 6, 3);
			SetFaceView(2, 3, 6, 4, 1, 3);
			SetFaceView(2, 4, 1, 4, 5, 3);
			SetFaceView(3, 1, 5, 3, 4, 1);
			SetFaceView(3, 2, 4, 2, 6, 4);
			SetFaceView(3, 3, 6, 1, 2, 3);
			SetFaceView(3, 4, 2, 4, 5, 2);
			SetFaceView(4, 1, 5, 2, 1, 1);
			SetFaceView(4, 2, 1, 2, 6, 1);
			SetFaceView(4, 3, 6, 2, 3, 3);
			SetFaceView(4, 4, 3, 4, 5, 1);
			SetFaceView(5, 1, 3, 3, 2, 2);
			SetFaceView(5, 2, 2, 3, 1, 2);
			SetFaceView(5, 3, 1, 3, 4, 2);
			SetFaceView(5, 4, 4, 3, 3, 2);
			SetFaceView(6, 1, 1, 1, 2, 4);
			SetFaceView(6, 2, 2, 1, 3, 4);
			SetFaceView(6, 3, 3, 1, 4, 4);
			SetFaceView(6, 4, 4, 1, 1, 4);
			// set transformations
			tTransform = new Transform[6, 4, 3, 6, 6];
			SetTransform(1, 1, Axis.X, Slice.Top, Direction.Left, Axis.Y, Slice.Left, Direction.Up, Axis.Z, Slice.Front, Direction.CCW);
			SetTransform(1, 2, Axis.Y, Slice.Right, Direction.Up, Axis.X, Slice.Top, Direction.Right, Axis.Z, Slice.Front, Direction.CCW);
			SetTransform(1, 3, Axis.X, Slice.Bottom, Direction.Right, Axis.Y, Slice.Right, Direction.Down, Axis.Z, Slice.Front, Direction.CCW);
			SetTransform(1, 4, Axis.Y, Slice.Left, Direction.Down, Axis.X, Slice.Bottom, Direction.Left, Axis.Z, Slice.Front, Direction.CCW);
			SetTransform(2, 1, Axis.X, Slice.Top, Direction.Left, Axis.Z, Slice.Front, Direction.CCW, Axis.Y, Slice.Right, Direction.Down);
			SetTransform(2, 2, Axis.Z, Slice.Back, Direction.CCW, Axis.X, Slice.Top, Direction.Right, Axis.Y, Slice.Right, Direction.Down);
			SetTransform(2, 3, Axis.X, Slice.Bottom, Direction.Right, Axis.Z, Slice.Back, Direction.CCW, Axis.Y, Slice.Right, Direction.Down);
			SetTransform(2, 4, Axis.Z, Slice.Front, Direction.CW, Axis.X, Slice.Bottom, Direction.Left, Axis.Y, Slice.Right, Direction.Down);
			SetTransform(3, 1, Axis.X, Slice.Top, Direction.Left, Axis.Y, Slice.Right, Direction.Down, Axis.Z, Slice.Back, Direction.CW);
			SetTransform(3, 2, Axis.Y, Slice.Left, Direction.Down, Axis.X, Slice.Top, Direction.Right, Axis.Z, Slice.Back, Direction.CW);
			SetTransform(3, 3, Axis.X, Slice.Bottom, Direction.Right, Axis.Y, Slice.Left, Direction.Up, Axis.Z, Slice.Back, Direction.CW);
			SetTransform(3, 4, Axis.Y, Slice.Right, Direction.Up, Axis.X, Slice.Bottom, Direction.Left, Axis.Z, Slice.Back, Direction.CW);
			SetTransform(4, 1, Axis.X, Slice.Top, Direction.Left, Axis.Z, Slice.Back, Direction.CW, Axis.Y, Slice.Left, Direction.Up);
			SetTransform(4, 2, Axis.Z, Slice.Front, Direction.CW, Axis.X, Slice.Top, Direction.Right, Axis.Y, Slice.Left, Direction.Up);
			SetTransform(4, 3, Axis.X, Slice.Bottom, Direction.Right, Axis.Z, Slice.Front, Direction.CCW, Axis.Y, Slice.Left, Direction.Up);
			SetTransform(4, 4, Axis.Z, Slice.Back, Direction.CCW, Axis.X, Slice.Bottom, Direction.Left, Axis.Y, Slice.Left, Direction.Up);
			SetTransform(5, 1, Axis.Z, Slice.Back, Direction.CCW, Axis.Y, Slice.Left, Direction.Up, Axis.X, Slice.Top, Direction.Right);
			SetTransform(5, 2, Axis.Y, Slice.Right, Direction.Up, Axis.Z, Slice.Back, Direction.CW, Axis.X, Slice.Top, Direction.Right);
			SetTransform(5, 3, Axis.Z, Slice.Front, Direction.CW, Axis.Y, Slice.Right, Direction.Down, Axis.X, Slice.Top, Direction.Right);
			SetTransform(5, 4, Axis.Y, Slice.Left, Direction.Down, Axis.Z, Slice.Front, Direction.CCW, Axis.X, Slice.Top, Direction.Right);
			SetTransform(6, 1, Axis.Z, Slice.Front, Direction.CW, Axis.Y, Slice.Left, Direction.Up, Axis.X, Slice.Bottom, Direction.Left);
			SetTransform(6, 2, Axis.Y, Slice.Right, Direction.Up, Axis.Z, Slice.Front, Direction.CCW, Axis.X, Slice.Bottom, Direction.Left);
			SetTransform(6, 3, Axis.Z, Slice.Back, Direction.CCW, Axis.Y, Slice.Right, Direction.Down, Axis.X, Slice.Bottom, Direction.Left);
			SetTransform(6, 4, Axis.Y, Slice.Left, Direction.Down, Axis.Z, Slice.Back, Direction.CW, Axis.X, Slice.Bottom, Direction.Left);
			// create cube with starting conditions
			Reset();
		}

		public void SetTransform(int face, int rotation, Axis aaX, Slice saX, Direction daX, Axis aaY, Slice saY, Direction daY, Axis aaZ, Slice saZ, Direction daZ) {
			// specify which transform to apply to each face
			Slice soX = ReverseSlice(saX);
			Slice soY = ReverseSlice(saY);
			Slice soZ = ReverseSlice(saZ);
			Direction doX = ReverseDirection(daX);
			Direction doY = ReverseDirection(daY);
			Direction doZ = ReverseDirection(daZ);
			tTransform[face - 1, rotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Left] = new Transform(aaX, saX, daX);
			tTransform[face - 1, rotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Right] = new Transform(aaX, saX, doX);
			tTransform[face - 1, rotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Left] = new Transform(aaX, soX, daX);
			tTransform[face - 1, rotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Right] = new Transform(aaX, soX, doX);
			tTransform[face - 1, rotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Up] = new Transform(aaY, saY, daY);
			tTransform[face - 1, rotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Down] = new Transform(aaY, saY, doY);
			tTransform[face - 1, rotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Up] = new Transform(aaY, soY, daY);
			tTransform[face - 1, rotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Down] = new Transform(aaY, soY, doY);
			tTransform[face - 1, rotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CCW] = new Transform(aaZ, saZ, daZ);
			tTransform[face - 1, rotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CW] = new Transform(aaZ, saZ, doZ);
			tTransform[face - 1, rotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CCW] = new Transform(aaZ, soZ, daZ);
			tTransform[face - 1, rotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CW] = new Transform(aaZ, soZ, doZ);
		}

		public void SetRotateFaceView(int face, int rotation, Axis axis, Direction direction, int newface, int newrotation) {
			iRotateFaceView[face - 1, rotation - 1, (int) axis, (int) direction] = newface;
			iRotateFaceRotation[face - 1, rotation - 1, (int) axis, (int) direction] = newrotation;
		}

		public void SetFaceView(int face, int rotation, int topface, int toprotation, int rightface, int rightrotation) {
			iTopFaceView[face - 1, rotation - 1] = topface;
			iTopFaceRotation[face - 1, rotation - 1] = toprotation;
			iRightFaceView[face - 1, rotation - 1] = rightface;
			iRightFaceRotation[face - 1, rotation - 1] = rightrotation;
		}

		public Direction ReverseDirection(Direction d) {
			if (d == Direction.CCW) return Direction.CW;
			if (d == Direction.CW) return Direction.CCW;
			if (d == Direction.Down) return Direction.Up;
			if (d == Direction.Up) return Direction.Down;
			if (d == Direction.Left) return Direction.Right;
			if (d == Direction.Right) return Direction.Left;
			return Direction.Left;
		}
		public Slice ReverseSlice(Slice s) {
			if (s == Slice.Back) return Slice.Front;
			if (s == Slice.Front) return Slice.Back;
			if (s == Slice.Top) return Slice.Bottom;
			if (s == Slice.Bottom) return Slice.Top;
			if (s == Slice.Left) return Slice.Right;
			if (s == Slice.Right) return Slice.Left;
			return Slice.Back;
		}

		#endregion

		#region Cube manipulation functions

		public void Reset() {
			// create block colors
			iFace = new int[6,9];
			for (int i = 1; i <= 6; i++) {
				for (int j = 0; j < 9; j++) {
					iFace[i-1,j] = i;
				}
			}

			// move to starting view
			iFrontFaceView = 1;
			iFrontFaceRotation = 1;
		}

		public void RotateView(Axis axis, Direction direction) {
			int iNewFace = iRotateFaceView[iFrontFaceView - 1, iFrontFaceRotation - 1, (int) axis, (int) direction];
			int iNewRotation = iRotateFaceRotation[iFrontFaceView - 1, iFrontFaceRotation - 1, (int) axis, (int) direction];
			iFrontFaceView = iNewFace;
			iFrontFaceRotation = iNewRotation;
		}

		public static void Transform(int[,] iFace, Transform transform) {
			switch (transform.axis) {
				case Axis.X:
					switch (transform.slice) {
						case Slice.Top:
							RotatePositions(iFace, 1, 1, false, 2, 1, false, 3, 1, false, 4, 1, false, transform.direction == Direction.Right ? false : true);
							RotateFace(iFace, 5, transform.direction == Direction.Right ? false : true);
							break;
						case Slice.Bottom:
							RotatePositions(iFace, 1, 3, false, 2, 3, false, 3, 3, false, 4, 3, false, transform.direction == Direction.Right ? false : true);
							RotateFace(iFace, 6, transform.direction == Direction.Right ? true : false);
							break;
						default:break;
					}
					break;
				case Axis.Y:
					switch (transform.slice) {
						case Slice.Left:
							RotatePositions(iFace, 1, 4, false, 5, 4, false, 3, 6, true, 6, 4, false, transform.direction == Direction.Up ? false : true);
							RotateFace(iFace, 4, transform.direction == Direction.Up ? false : true);
							break;
						case Slice.Right:
							RotatePositions(iFace, 1, 6, false, 5, 6, false, 3, 4, true, 6, 6, false, transform.direction == Direction.Up ? false : true);
							RotateFace(iFace, 2, transform.direction == Direction.Up ? true : false);
							break;
						default:break;
					}
					break;
				case Axis.Z:
					switch (transform.slice) {
						case Slice.Front:
							RotatePositions(iFace, 5, 3, false, 4, 6, true, 6, 1, true, 2, 4, false, transform.direction == Direction.CCW ? false : true);
							RotateFace(iFace, 1, transform.direction == Direction.CCW ? false : true);
							break;
						case Slice.Back:
							RotatePositions(iFace, 5, 1, false, 4, 4, true, 6, 3, true, 2, 6, false, transform.direction == Direction.CCW ? false : true);
							RotateFace(iFace, 3, transform.direction == Direction.CCW ? true : false);
							break;
						default:break;
					}
					break;
				default: break;
			}
		}

		public static void RotateFace(int[,] iFace, int face, bool direction) {
			int[,] swap = new int[4,3];
			FetchValues(iFace, face - 1, 1, out swap[0,0], out swap[0,1], out swap[0,2]);
			FetchValues(iFace, face - 1, 6, out swap[1,0], out swap[1,1], out swap[1,2]);
			FetchValues(iFace, face - 1, 3, out swap[2,2], out swap[2,1], out swap[2,0]);
			FetchValues(iFace, face - 1, 4, out swap[3,2], out swap[3,1], out swap[3,0]);
			if (direction == true) {
				SetValues(iFace, face - 1, 6, swap[0,0], swap[0,1], swap[0,2]);
				SetValues(iFace, face - 1, 3, swap[1,2], swap[1,1], swap[1,0]);
				SetValues(iFace, face - 1, 4, swap[2,2], swap[2,1], swap[2,0]);
				SetValues(iFace, face - 1, 1, swap[3,0], swap[3,1], swap[3,2]);
			} else {
				SetValues(iFace, face - 1, 4, swap[0,2], swap[0,1], swap[0,0]);
				SetValues(iFace, face - 1, 1, swap[1,0], swap[1,1], swap[1,2]);
				SetValues(iFace, face - 1, 6, swap[2,0], swap[2,1], swap[2,2]);
				SetValues(iFace, face - 1, 3, swap[3,2], swap[3,1], swap[3,0]);
			}
		}

		public static void RotatePositions(int[,] iFace, int face1, int rc1, bool inv1, int face2, int rc2, bool inv2, int face3, int rc3, bool inv3, int face4, int rc4, bool inv4, bool direction) {
			int[,] swap = new int[4,3];
			if (inv1 == false)
				FetchValues(iFace, face1 - 1, rc1, out swap[0,0], out swap[0,1], out swap[0,2]);
			else
				FetchValues(iFace, face1 - 1, rc1, out swap[0,2], out swap[0,1], out swap[0,0]);
			if (inv2 == false)
				FetchValues(iFace, face2 - 1, rc2, out swap[1,0], out swap[1,1], out swap[1,2]);
			else
				FetchValues(iFace, face2 - 1, rc2, out swap[1,2], out swap[1,1], out swap[1,0]);
			if (inv3 == false)
				FetchValues(iFace, face3 - 1, rc3, out swap[2,0], out swap[2,1], out swap[2,2]);
			else
				FetchValues(iFace, face3 - 1, rc3, out swap[2,2], out swap[2,1], out swap[2,0]);
			if (inv4 == false)
				FetchValues(iFace, face4 - 1, rc4, out swap[3,0], out swap[3,1], out swap[3,2]);
			else
				FetchValues(iFace, face4 - 1, rc4, out swap[3,2], out swap[3,1], out swap[3,0]);
			if (direction == false) {
				if (inv2 == false)
					SetValues(iFace, face2 - 1, rc2, swap[0,0], swap[0,1], swap[0,2]);
				else
					SetValues(iFace, face2 - 1, rc2, swap[0,2], swap[0,1], swap[0,0]);
				if (inv3 == false)
					SetValues(iFace, face3 - 1, rc3, swap[1,0], swap[1,1], swap[1,2]);
				else
					SetValues(iFace, face3 - 1, rc3, swap[1,2], swap[1,1], swap[1,0]);
				if (inv4 == false)
					SetValues(iFace, face4 - 1, rc4, swap[2,0], swap[2,1], swap[2,2]);
				else
					SetValues(iFace, face4 - 1, rc4, swap[2,2], swap[2,1], swap[2,0]);
				if (inv1 == false)
					SetValues(iFace, face1 - 1, rc1, swap[3,0], swap[3,1], swap[3,2]);
				else
					SetValues(iFace, face1 - 1, rc1, swap[3,2], swap[3,1], swap[3,0]);
			} else {
				if (inv4 == false)
					SetValues(iFace, face4 - 1, rc4, swap[0,0], swap[0,1], swap[0,2]);
				else
					SetValues(iFace, face4 - 1, rc4, swap[0,2], swap[0,1], swap[0,0]);
				if (inv1 == false)
					SetValues(iFace, face1 - 1, rc1, swap[1,0], swap[1,1], swap[1,2]);
				else
					SetValues(iFace, face1 - 1, rc1, swap[1,2], swap[1,1], swap[1,0]);
				if (inv2 == false)
					SetValues(iFace, face2 - 1, rc2, swap[2,0], swap[2,1], swap[2,2]);
				else
					SetValues(iFace, face2 - 1, rc2, swap[2,2], swap[2,1], swap[2,0]);
				if (inv3 == false)
					SetValues(iFace, face3 - 1, rc3, swap[3,0], swap[3,1], swap[3,2]);
				else
					SetValues(iFace, face3 - 1, rc3, swap[3,2], swap[3,1], swap[3,0]);
			}
		}

		#endregion

		#region Cube analysis functions

		public static void FetchValues(int[,] iFace, int face, int rc, out int v1, out int v2, out int v3) {
			// Given a face and a row or column counter, this function returns the values of the three cubes in that row or column.
			switch (rc) {
				case 1:
					v1 = iFace[face, 0];
					v2 = iFace[face, 1];
					v3 = iFace[face, 2];
					break;
				case 2:
					v1 = iFace[face, 3];
					v2 = iFace[face, 4];
					v3 = iFace[face, 5];
					break;
				case 3:
					v1 = iFace[face, 6];
					v2 = iFace[face, 7];
					v3 = iFace[face, 8];
					break;
				case 4:
					v1 = iFace[face, 0];
					v2 = iFace[face, 3];
					v3 = iFace[face, 6];
					break;
				case 5:
					v1 = iFace[face, 1];
					v2 = iFace[face, 4];
					v3 = iFace[face, 7];
					break;
				case 6:
					v1 = iFace[face, 2];
					v2 = iFace[face, 5];
					v3 = iFace[face, 8];
					break;
				default:
					v1 = v2 = v3 = 0;
					break;
			}
		}

		public static void SetValues(int[,] iFace, int face, int rc, int v1, int v2, int v3) {
			// Given a face and row or column, this function sets the values of the cubes in that row or column.
			switch (rc) {
				case 1:
					iFace[face, 0] = v1;
					iFace[face, 1] = v2;
					iFace[face, 2] = v3;
					break;
				case 2:
					iFace[face, 3] = v1;
					iFace[face, 4] = v2;
					iFace[face, 5] = v3;
					break;
				case 3:
					iFace[face, 6] = v1;
					iFace[face, 7] = v2;
					iFace[face, 8] = v3;
					break;
				case 4:
					iFace[face, 0] = v1;
					iFace[face, 3] = v2;
					iFace[face, 6] = v3;
					break;
				case 5:
					iFace[face, 1] = v1;
					iFace[face, 4] = v2;
					iFace[face, 7] = v3;
					break;
				case 6:
					iFace[face, 2] = v1;
					iFace[face, 5] = v2;
					iFace[face, 8] = v3;
					break;
				default:
					break;
			}
		}

		public static int CalculateScore(int[,] iFace) {
			// This is the heuristic function. 
			int score = 120;
			for (int i = 0; i < 6; i++) {
				if (iFace[i,0] == iFace[i,1])
					score--;
				if (iFace[i,0] == iFace[i,3])
					score--;
				if (iFace[i,2] == iFace[i,1])
					score--;
				if (iFace[i,2] == iFace[i,5])
					score--;
				if (iFace[i,6] == iFace[i,3])
					score--;
				if (iFace[i,6] == iFace[i,7])
					score--;
				if (iFace[i,8] == iFace[i,7])
					score--;
				if (iFace[i,8] == iFace[i,5])
					score--;
				if (iFace[i,4] == iFace[i,0])
					score--;
				if (iFace[i,4] == iFace[i,1])
					score -= 2;
				if (iFace[i,4] == iFace[i,2])
					score--;
				if (iFace[i,4] == iFace[i,3])
					score -= 2;
				if (iFace[i,4] == iFace[i,5])
					score -= 2;
				if (iFace[i,4] == iFace[i,6])
					score--;
				if (iFace[i,4] == iFace[i,7])
					score -= 2;
				if (iFace[i,4] == iFace[i,8])
					score--;
			}
			return score;
		}

		public int[] ComputeFaceValues(int face, int rotation) {
			// This function computes the score of a state of the cube.
			int[] result = new int[9];
			// copy values
			for (int i = 0; i < 9; i++)
				result[i] = iFace[face - 1, i];
			// rotate
			for (int i = 2; i <= rotation; i++) {
				int swap = result[0];
				result[0] = result[2];
				result[2] = result[8];
				result[8] = result[6];
				result[6] = swap;
				swap = result[1];
				result[1] = result[5];
				result[5] = result[7];
				result[7] = result[3];
				result[3] = swap;
			}
			return result;
		}

		public void Solve() {
			// This function is the core of the solution finder.
			bStop = bDone = false;
			BinaryPriorityQueue heap = new BinaryPriorityQueue(new SolutionStepComparer());
			Hashtable hash = new Hashtable();
			// create starting solution, and add to heap and hashtable
			SolutionStep start = new SolutionStep(iFace, 0, null, null);
			heap.Push(start);
			hash.Add(start.strHashcode, null);
			// start the loop
			int iCount = 0;
			int iThresholdScore = 1000;
			solution = null;
			while ((heap.Count > 0) && (bStop == false)) {
				SolutionStep step = (SolutionStep) heap.Pop();
				if (step.bSolved == true) {  // this step is a winner! - return it.
					strStatus = "Status: Found solution after " + iCount + " searches.";
					solution = step;
					break;
				}
				// test and add all children
				iThresholdScore = 250 - hash.Count / (1000 * iThreshold);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.X, Slice.Top, Direction.Left), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.X, Slice.Top, Direction.Right), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.X, Slice.Bottom, Direction.Left), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.X, Slice.Bottom, Direction.Right), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.Y, Slice.Left, Direction.Up), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.Y, Slice.Left, Direction.Down), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.Y, Slice.Right, Direction.Up), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.Y, Slice.Right, Direction.Down), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.Z, Slice.Front, Direction.CCW), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.Z, Slice.Front, Direction.CW), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.Z, Slice.Back, Direction.CCW), step), heap, hash, iThresholdScore);
				TestAndAdd(new SolutionStep(step.iFace, step.iStepNumber + 1, new Transform(Axis.Z, Slice.Back, Direction.CW), step), heap, hash, iThresholdScore);
				iCount++;
				strStatus = "Status: Searched " + iCount + " states; " + heap.Count + " remaining.";
			}
			if (bStop == true)
				strStatus = "Status: Aborted.";
			else if (solution == null)
				strStatus = "Status: No solution found.";
			bDone = true;
		}

		public bool TestAndAdd(SolutionStep step, BinaryPriorityQueue heap, Hashtable hash, int iThresholdScore) {
			// This function checks the search hashtable to see if this state has already been explored. If not, the state is added to the heap and registered in the hashtable.
			if ((hash.ContainsKey(step.strHashcode) == false) && (step.iScore < iThresholdScore)) {
				hash.Add(step.strHashcode, null);
				heap.Push(step);
				return true;
			}
			else
				return false;
		}

		#endregion

	}

	#region Enums and Data Structures

	public class SolutionStepComparer : IComparer {
		// This class compares two solutions for the purpose of sorting the heap (priority queue) by score.
		public int Compare(object x, object y) {
			if ((x is SolutionStep) && (y is SolutionStep))
				return (((SolutionStep) x).iScore < ((SolutionStep) y).iScore ? -1 : ((SolutionStep) x).iScore > ((SolutionStep) y).iScore ? +1 : 0);
			else
				return 0;
		}
	}

	public class SolutionStep {
		// This class represents a state of the cube in a solution chain.
		public int[,] iFace;
		public Transform transform;
		public int iStepNumber;
		public int iScore;
		public bool bSolved;
		public SolutionStep parent;
		public string strHashcode;
		public SolutionStep(int[,] iFace, int iStepNumber, Transform transform, SolutionStep parent) {
			this.iFace = new int[6,9];
			for (int i =0; i < 6; i++) {
				for (int j = 0; j < 9; j++) {
					this.iFace[i, j] = iFace[i, j];
				}
			}
			if (transform !=null)
				Cube.Transform(this.iFace, transform);
			System.Text.StringBuilder strHashcode = new System.Text.StringBuilder();
			for (int i = 0; i < 6; i++) {
				for (int j = 0; j < 9; j++) {
					strHashcode.Append(this.iFace[i, j]);
				}
			}
			this.strHashcode = strHashcode.ToString();
			this.bSolved = (this.strHashcode == "111111111222222222333333333444444444555555555666666666" ? true : false);
			this.iStepNumber = iStepNumber;
			this.iScore = Cube.CalculateScore(this.iFace) + iStepNumber;
			this.transform = transform;
			this.parent = parent;
		}
	}

	public enum Axis { X = 0, Y, Z };
	public enum Slice { Left = 0, Right, Top, Bottom, Front, Back };
	public enum Direction { Left = 0, Right, Down, Up, CCW, CW };
	public class Transform {
		// This class represents a manipulation of the cube by rotating a particular slice.
		public Axis axis;
		public Slice slice;
		public Direction direction;
		public Transform(Axis a, Slice s, Direction d) {
			axis =a ;
			slice = s;
			direction = d;
		}
	}

	#endregion

}
