// CubeSearch v1.0
// Written by David J. Stein, Esq. See "License.html" for information on terms and conditions of use.
//
// This module contains the code for presenting, displaying, and operating the cube and user form.

#region Using declarations

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;

#endregion

namespace Cube_Search {

	public class CubeSearch : System.Windows.Forms.Form {

		#region Variables, initialization code, and Main

		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.GroupBox groupBox3;
		private System.Windows.Forms.Button btnReset;
		private System.Windows.Forms.Button btnRandom;
		private System.Windows.Forms.PictureBox pictureBox2;
		private System.Windows.Forms.PictureBox pictureBox3;
		private System.Windows.Forms.PictureBox pictureBox4;
		private System.Windows.Forms.PictureBox pictureBox5;
		private System.Windows.Forms.PictureBox pictureBox6;
		private System.Windows.Forms.PictureBox pictureBox7;
		private System.Windows.Forms.GroupBox groupBox4;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.ColumnHeader StepColumn;
		private System.Windows.Forms.ColumnHeader ActionColumn;
		private System.Windows.Forms.ColumnHeader ScoreColumn;
		private System.Windows.Forms.Label label56;
		private System.Windows.Forms.Label label57;
		private System.Windows.Forms.Label label58;
		private System.Windows.Forms.Label label59;
		private System.Windows.Forms.Label label60;
		private System.Windows.Forms.Label label61;
		private System.Windows.Forms.Button btnRotateZCW;
		private System.Windows.Forms.Button btnRotateZCCCW;
		private System.Windows.Forms.Button btnRotateYUp;
		private System.Windows.Forms.Button btnRotateYDown;
		private System.Windows.Forms.Button btnRotateXRight;
		private System.Windows.Forms.Button btnRotateXLeft;
		private System.Windows.Forms.Button btnSliceYRightDown;
		private System.Windows.Forms.Button btnSliceYRightUp;
		private System.Windows.Forms.Button btnSliceYLeftDown;
		private System.Windows.Forms.Button btnSliceYLeftUp;
		private System.Windows.Forms.Button btnSliceZBackCW;
		private System.Windows.Forms.Button btnSliceZBackCCW;
		private System.Windows.Forms.Button btnSliceZFrontCW;
		private System.Windows.Forms.Button btnSliceZFrontCCW;
		private System.Windows.Forms.Button btnSliceXBottomRight;
		private System.Windows.Forms.Button btnSliceXBottomLeft;
		private System.Windows.Forms.Button btnSliceXTopRight;
		private System.Windows.Forms.Button btnSliceXTopLeft;
		private System.Windows.Forms.Button btnSolve;
		private System.Windows.Forms.ListView lstSolution;
		private System.Windows.Forms.Label txtFace69;
		private System.Windows.Forms.Label txtFace68;
		private System.Windows.Forms.Label txtFace67;
		private System.Windows.Forms.Label txtFace66;
		private System.Windows.Forms.Label txtFace65;
		private System.Windows.Forms.Label txtFace64;
		private System.Windows.Forms.Label txtFace63;
		private System.Windows.Forms.Label txtFace62;
		private System.Windows.Forms.Label txtFace61;
		private System.Windows.Forms.Label txtFace59;
		private System.Windows.Forms.Label txtFace58;
		private System.Windows.Forms.Label txtFace57;
		private System.Windows.Forms.Label txtFace56;
		private System.Windows.Forms.Label txtFace55;
		private System.Windows.Forms.Label txtFace54;
		private System.Windows.Forms.Label txtFace53;
		private System.Windows.Forms.Label txtFace52;
		private System.Windows.Forms.Label txtFace51;
		private System.Windows.Forms.Label txtFace49;
		private System.Windows.Forms.Label txtFace48;
		private System.Windows.Forms.Label txtFace47;
		private System.Windows.Forms.Label txtFace46;
		private System.Windows.Forms.Label txtFace45;
		private System.Windows.Forms.Label txtFace44;
		private System.Windows.Forms.Label txtFace43;
		private System.Windows.Forms.Label txtFace42;
		private System.Windows.Forms.Label txtFace41;
		private System.Windows.Forms.Label txtFace39;
		private System.Windows.Forms.Label txtFace38;
		private System.Windows.Forms.Label txtFace37;
		private System.Windows.Forms.Label txtFace36;
		private System.Windows.Forms.Label txtFace35;
		private System.Windows.Forms.Label txtFace34;
		private System.Windows.Forms.Label txtFace33;
		private System.Windows.Forms.Label txtFace32;
		private System.Windows.Forms.Label txtFace31;
		private System.Windows.Forms.Label txtFace29;
		private System.Windows.Forms.Label txtFace28;
		private System.Windows.Forms.Label txtFace27;
		private System.Windows.Forms.Label txtFace26;
		private System.Windows.Forms.Label txtFace25;
		private System.Windows.Forms.Label txtFace23;
		private System.Windows.Forms.Label txtFace22;
		private System.Windows.Forms.Label txtFace21;
		private System.Windows.Forms.Label txtFace19;
		private System.Windows.Forms.Label txtFace18;
		private System.Windows.Forms.Label txtFace17;
		private System.Windows.Forms.Label txtFace16;
		private System.Windows.Forms.Label txtFace15;
		private System.Windows.Forms.Label txtFace14;
		private System.Windows.Forms.Label txtFace13;
		private System.Windows.Forms.Label txtFace12;
		private System.Windows.Forms.Label txtFace11;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.PictureBox picCube;
		private System.Windows.Forms.Label txtFace24;
		private System.Windows.Forms.ListBox lstColorSchemes;
		private System.Windows.Forms.TextBox txtStatus;
		private System.Windows.Forms.Timer timerUpdate;
		private System.Windows.Forms.TrackBar trackThreshold;
		private System.Windows.Forms.GroupBox groupBox5;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.MainMenu menuMain;
		private System.Windows.Forms.MenuItem menuHelpAbout;
		private System.Windows.Forms.MenuItem menuExit;
		private System.Windows.Forms.MenuItem menuItem1;
		private Cube cube;  // an instance of the cube class
		private Color[] colors;  // the current color scheme (one color for each of six faces)
		private int iMouseSpot = 0;  // the aspect of the cube being controlled by the mouse
		private int iRotated = 0;  // whether or not the aspect of the cube has been rotated (-1 = left, +1 = right, 0 = no rotation)
		private int iMouseX, iMouseY;  // mouse coordinates
		private bool bMouseDown = false;  // flag for whether or not the mouse button is down

		public CubeSearch() {
			InitializeComponent();
			cube = new Cube();
			// create colors and initial color scheme
			colors = new Color[6];
			colors[0] = Color.Red;
			colors[1] = Color.Lime;
			colors[2] = Color.Orange;
			colors[3] = Color.Blue;
			colors[4] = Color.Yellow;
			colors[5] = Color.White;
			UpdateConfiguration();  // draw 2D configuration structure
		}

		#region Uninteresting Windows Form Designer generated code
		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(CubeSearch));
			this.picCube = new System.Windows.Forms.PictureBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.label1 = new System.Windows.Forms.Label();
			this.lstColorSchemes = new System.Windows.Forms.ListBox();
			this.btnRotateZCW = new System.Windows.Forms.Button();
			this.btnRotateZCCCW = new System.Windows.Forms.Button();
			this.btnRotateYUp = new System.Windows.Forms.Button();
			this.btnRotateXRight = new System.Windows.Forms.Button();
			this.btnRotateXLeft = new System.Windows.Forms.Button();
			this.btnRotateYDown = new System.Windows.Forms.Button();
			this.pictureBox7 = new System.Windows.Forms.PictureBox();
			this.pictureBox6 = new System.Windows.Forms.PictureBox();
			this.pictureBox5 = new System.Windows.Forms.PictureBox();
			this.pictureBox4 = new System.Windows.Forms.PictureBox();
			this.pictureBox3 = new System.Windows.Forms.PictureBox();
			this.pictureBox2 = new System.Windows.Forms.PictureBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.btnSliceYRightDown = new System.Windows.Forms.Button();
			this.btnSliceYLeftDown = new System.Windows.Forms.Button();
			this.btnSliceZBackCW = new System.Windows.Forms.Button();
			this.btnSliceZBackCCW = new System.Windows.Forms.Button();
			this.btnSliceZFrontCW = new System.Windows.Forms.Button();
			this.btnSliceZFrontCCW = new System.Windows.Forms.Button();
			this.btnSliceXBottomRight = new System.Windows.Forms.Button();
			this.btnSliceXBottomLeft = new System.Windows.Forms.Button();
			this.btnSliceXTopRight = new System.Windows.Forms.Button();
			this.btnSliceXTopLeft = new System.Windows.Forms.Button();
			this.btnSliceYLeftUp = new System.Windows.Forms.Button();
			this.btnSliceYRightUp = new System.Windows.Forms.Button();
			this.btnRandom = new System.Windows.Forms.Button();
			this.btnReset = new System.Windows.Forms.Button();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.groupBox5 = new System.Windows.Forms.GroupBox();
			this.label3 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.trackThreshold = new System.Windows.Forms.TrackBar();
			this.txtStatus = new System.Windows.Forms.TextBox();
			this.btnSolve = new System.Windows.Forms.Button();
			this.lstSolution = new System.Windows.Forms.ListView();
			this.StepColumn = new System.Windows.Forms.ColumnHeader();
			this.ActionColumn = new System.Windows.Forms.ColumnHeader();
			this.ScoreColumn = new System.Windows.Forms.ColumnHeader();
			this.groupBox4 = new System.Windows.Forms.GroupBox();
			this.label61 = new System.Windows.Forms.Label();
			this.label60 = new System.Windows.Forms.Label();
			this.label59 = new System.Windows.Forms.Label();
			this.label58 = new System.Windows.Forms.Label();
			this.label57 = new System.Windows.Forms.Label();
			this.label56 = new System.Windows.Forms.Label();
			this.txtFace69 = new System.Windows.Forms.Label();
			this.txtFace68 = new System.Windows.Forms.Label();
			this.txtFace67 = new System.Windows.Forms.Label();
			this.txtFace66 = new System.Windows.Forms.Label();
			this.txtFace65 = new System.Windows.Forms.Label();
			this.txtFace64 = new System.Windows.Forms.Label();
			this.txtFace63 = new System.Windows.Forms.Label();
			this.txtFace62 = new System.Windows.Forms.Label();
			this.txtFace61 = new System.Windows.Forms.Label();
			this.txtFace59 = new System.Windows.Forms.Label();
			this.txtFace58 = new System.Windows.Forms.Label();
			this.txtFace57 = new System.Windows.Forms.Label();
			this.txtFace56 = new System.Windows.Forms.Label();
			this.txtFace55 = new System.Windows.Forms.Label();
			this.txtFace54 = new System.Windows.Forms.Label();
			this.txtFace53 = new System.Windows.Forms.Label();
			this.txtFace52 = new System.Windows.Forms.Label();
			this.txtFace51 = new System.Windows.Forms.Label();
			this.txtFace49 = new System.Windows.Forms.Label();
			this.txtFace48 = new System.Windows.Forms.Label();
			this.txtFace47 = new System.Windows.Forms.Label();
			this.txtFace46 = new System.Windows.Forms.Label();
			this.txtFace45 = new System.Windows.Forms.Label();
			this.txtFace44 = new System.Windows.Forms.Label();
			this.txtFace43 = new System.Windows.Forms.Label();
			this.txtFace42 = new System.Windows.Forms.Label();
			this.txtFace41 = new System.Windows.Forms.Label();
			this.txtFace39 = new System.Windows.Forms.Label();
			this.txtFace38 = new System.Windows.Forms.Label();
			this.txtFace37 = new System.Windows.Forms.Label();
			this.txtFace36 = new System.Windows.Forms.Label();
			this.txtFace35 = new System.Windows.Forms.Label();
			this.txtFace34 = new System.Windows.Forms.Label();
			this.txtFace33 = new System.Windows.Forms.Label();
			this.txtFace32 = new System.Windows.Forms.Label();
			this.txtFace31 = new System.Windows.Forms.Label();
			this.txtFace29 = new System.Windows.Forms.Label();
			this.txtFace28 = new System.Windows.Forms.Label();
			this.txtFace27 = new System.Windows.Forms.Label();
			this.txtFace26 = new System.Windows.Forms.Label();
			this.txtFace25 = new System.Windows.Forms.Label();
			this.txtFace24 = new System.Windows.Forms.Label();
			this.txtFace23 = new System.Windows.Forms.Label();
			this.txtFace22 = new System.Windows.Forms.Label();
			this.txtFace21 = new System.Windows.Forms.Label();
			this.txtFace19 = new System.Windows.Forms.Label();
			this.txtFace18 = new System.Windows.Forms.Label();
			this.txtFace17 = new System.Windows.Forms.Label();
			this.txtFace16 = new System.Windows.Forms.Label();
			this.txtFace15 = new System.Windows.Forms.Label();
			this.txtFace14 = new System.Windows.Forms.Label();
			this.txtFace13 = new System.Windows.Forms.Label();
			this.txtFace12 = new System.Windows.Forms.Label();
			this.txtFace11 = new System.Windows.Forms.Label();
			this.timerUpdate = new System.Windows.Forms.Timer(this.components);
			this.menuMain = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.menuHelpAbout = new System.Windows.Forms.MenuItem();
			this.menuExit = new System.Windows.Forms.MenuItem();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.groupBox5.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.trackThreshold)).BeginInit();
			this.groupBox4.SuspendLayout();
			this.SuspendLayout();
			// 
			// picCube
			// 
			this.picCube.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picCube.Location = new System.Drawing.Point(8, 8);
			this.picCube.Name = "picCube";
			this.picCube.Size = new System.Drawing.Size(304, 304);
			this.picCube.TabIndex = 0;
			this.picCube.TabStop = false;
			this.picCube.Paint += new System.Windows.Forms.PaintEventHandler(this.picCube_Paint);
			this.picCube.MouseUp += new System.Windows.Forms.MouseEventHandler(this.picCube_MouseUp);
			this.picCube.MouseMove += new System.Windows.Forms.MouseEventHandler(this.picCube_MouseMove);
			this.picCube.MouseLeave += new System.EventHandler(this.picCube_MouseLeave);
			this.picCube.MouseDown += new System.Windows.Forms.MouseEventHandler(this.picCube_MouseDown);
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.label1);
			this.groupBox1.Controls.Add(this.lstColorSchemes);
			this.groupBox1.Controls.Add(this.btnRotateZCW);
			this.groupBox1.Controls.Add(this.btnRotateZCCCW);
			this.groupBox1.Controls.Add(this.btnRotateYUp);
			this.groupBox1.Controls.Add(this.btnRotateXRight);
			this.groupBox1.Controls.Add(this.btnRotateXLeft);
			this.groupBox1.Controls.Add(this.btnRotateYDown);
			this.groupBox1.Font = new System.Drawing.Font("Garamond", 11.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.groupBox1.Location = new System.Drawing.Point(8, 328);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(144, 304);
			this.groupBox1.TabIndex = 1;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "View";
			// 
			// label1
			// 
			this.label1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label1.Font = new System.Drawing.Font("Garamond", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label1.Location = new System.Drawing.Point(8, 224);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(128, 16);
			this.label1.TabIndex = 7;
			this.label1.Text = "Color Scheme";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// lstColorSchemes
			// 
			this.lstColorSchemes.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lstColorSchemes.ItemHeight = 12;
			this.lstColorSchemes.Items.AddRange(new object[] {
																				 "Original",
																				 "Reds",
																				 "Greens",
																				 "Blues",
																				 "Pastels",
																				 "Earth Tones",
																				 "Neon",
																				 "Greyscale"});
			this.lstColorSchemes.Location = new System.Drawing.Point(8, 240);
			this.lstColorSchemes.Name = "lstColorSchemes";
			this.lstColorSchemes.ScrollAlwaysVisible = true;
			this.lstColorSchemes.Size = new System.Drawing.Size(128, 52);
			this.lstColorSchemes.TabIndex = 6;
			this.lstColorSchemes.SelectedIndexChanged += new System.EventHandler(this.lstColorSchemes_SelectedIndexChanged);
			// 
			// btnRotateZCW
			// 
			this.btnRotateZCW.Image = ((System.Drawing.Image)(resources.GetObject("btnRotateZCW.Image")));
			this.btnRotateZCW.Location = new System.Drawing.Point(80, 152);
			this.btnRotateZCW.Name = "btnRotateZCW";
			this.btnRotateZCW.Size = new System.Drawing.Size(56, 56);
			this.btnRotateZCW.TabIndex = 5;
			this.btnRotateZCW.Click += new System.EventHandler(this.btnRotateZCW_Click);
			// 
			// btnRotateZCCCW
			// 
			this.btnRotateZCCCW.Image = ((System.Drawing.Image)(resources.GetObject("btnRotateZCCCW.Image")));
			this.btnRotateZCCCW.Location = new System.Drawing.Point(8, 152);
			this.btnRotateZCCCW.Name = "btnRotateZCCCW";
			this.btnRotateZCCCW.Size = new System.Drawing.Size(56, 56);
			this.btnRotateZCCCW.TabIndex = 4;
			this.btnRotateZCCCW.Click += new System.EventHandler(this.btnRotateZCCCW_Click);
			// 
			// btnRotateYUp
			// 
			this.btnRotateYUp.Image = ((System.Drawing.Image)(resources.GetObject("btnRotateYUp.Image")));
			this.btnRotateYUp.Location = new System.Drawing.Point(80, 88);
			this.btnRotateYUp.Name = "btnRotateYUp";
			this.btnRotateYUp.Size = new System.Drawing.Size(56, 56);
			this.btnRotateYUp.TabIndex = 3;
			this.btnRotateYUp.Click += new System.EventHandler(this.btnRotateYUp_Click);
			// 
			// btnRotateXRight
			// 
			this.btnRotateXRight.Image = ((System.Drawing.Image)(resources.GetObject("btnRotateXRight.Image")));
			this.btnRotateXRight.Location = new System.Drawing.Point(80, 24);
			this.btnRotateXRight.Name = "btnRotateXRight";
			this.btnRotateXRight.Size = new System.Drawing.Size(56, 56);
			this.btnRotateXRight.TabIndex = 1;
			this.btnRotateXRight.Click += new System.EventHandler(this.btnRotateXRight_Click);
			// 
			// btnRotateXLeft
			// 
			this.btnRotateXLeft.Image = ((System.Drawing.Image)(resources.GetObject("btnRotateXLeft.Image")));
			this.btnRotateXLeft.Location = new System.Drawing.Point(8, 24);
			this.btnRotateXLeft.Name = "btnRotateXLeft";
			this.btnRotateXLeft.Size = new System.Drawing.Size(56, 56);
			this.btnRotateXLeft.TabIndex = 0;
			this.btnRotateXLeft.Click += new System.EventHandler(this.btnRotateXLeft_Click);
			// 
			// btnRotateYDown
			// 
			this.btnRotateYDown.Image = ((System.Drawing.Image)(resources.GetObject("btnRotateYDown.Image")));
			this.btnRotateYDown.Location = new System.Drawing.Point(8, 88);
			this.btnRotateYDown.Name = "btnRotateYDown";
			this.btnRotateYDown.Size = new System.Drawing.Size(56, 56);
			this.btnRotateYDown.TabIndex = 2;
			this.btnRotateYDown.Click += new System.EventHandler(this.btnRotateYDown_Click);
			// 
			// pictureBox7
			// 
			this.pictureBox7.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pictureBox7.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox7.Image")));
			this.pictureBox7.Location = new System.Drawing.Point(56, 264);
			this.pictureBox7.Name = "pictureBox7";
			this.pictureBox7.Size = new System.Drawing.Size(32, 32);
			this.pictureBox7.TabIndex = 6;
			this.pictureBox7.TabStop = false;
			// 
			// pictureBox6
			// 
			this.pictureBox6.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pictureBox6.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox6.Image")));
			this.pictureBox6.Location = new System.Drawing.Point(56, 216);
			this.pictureBox6.Name = "pictureBox6";
			this.pictureBox6.Size = new System.Drawing.Size(32, 32);
			this.pictureBox6.TabIndex = 5;
			this.pictureBox6.TabStop = false;
			// 
			// pictureBox5
			// 
			this.pictureBox5.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pictureBox5.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox5.Image")));
			this.pictureBox5.Location = new System.Drawing.Point(56, 168);
			this.pictureBox5.Name = "pictureBox5";
			this.pictureBox5.Size = new System.Drawing.Size(32, 32);
			this.pictureBox5.TabIndex = 4;
			this.pictureBox5.TabStop = false;
			// 
			// pictureBox4
			// 
			this.pictureBox4.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pictureBox4.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox4.Image")));
			this.pictureBox4.Location = new System.Drawing.Point(56, 120);
			this.pictureBox4.Name = "pictureBox4";
			this.pictureBox4.Size = new System.Drawing.Size(32, 32);
			this.pictureBox4.TabIndex = 3;
			this.pictureBox4.TabStop = false;
			// 
			// pictureBox3
			// 
			this.pictureBox3.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pictureBox3.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox3.Image")));
			this.pictureBox3.Location = new System.Drawing.Point(56, 72);
			this.pictureBox3.Name = "pictureBox3";
			this.pictureBox3.Size = new System.Drawing.Size(32, 32);
			this.pictureBox3.TabIndex = 2;
			this.pictureBox3.TabStop = false;
			// 
			// pictureBox2
			// 
			this.pictureBox2.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pictureBox2.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox2.Image")));
			this.pictureBox2.Location = new System.Drawing.Point(56, 24);
			this.pictureBox2.Name = "pictureBox2";
			this.pictureBox2.Size = new System.Drawing.Size(32, 32);
			this.pictureBox2.TabIndex = 1;
			this.pictureBox2.TabStop = false;
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.btnSliceYRightDown);
			this.groupBox2.Controls.Add(this.btnSliceYLeftDown);
			this.groupBox2.Controls.Add(this.btnSliceZBackCW);
			this.groupBox2.Controls.Add(this.btnSliceZBackCCW);
			this.groupBox2.Controls.Add(this.btnSliceZFrontCW);
			this.groupBox2.Controls.Add(this.btnSliceZFrontCCW);
			this.groupBox2.Controls.Add(this.btnSliceXBottomRight);
			this.groupBox2.Controls.Add(this.btnSliceXBottomLeft);
			this.groupBox2.Controls.Add(this.btnSliceXTopRight);
			this.groupBox2.Controls.Add(this.btnSliceXTopLeft);
			this.groupBox2.Controls.Add(this.pictureBox2);
			this.groupBox2.Controls.Add(this.pictureBox3);
			this.groupBox2.Controls.Add(this.pictureBox4);
			this.groupBox2.Controls.Add(this.pictureBox5);
			this.groupBox2.Controls.Add(this.pictureBox6);
			this.groupBox2.Controls.Add(this.pictureBox7);
			this.groupBox2.Controls.Add(this.btnSliceYLeftUp);
			this.groupBox2.Controls.Add(this.btnSliceYRightUp);
			this.groupBox2.Font = new System.Drawing.Font("Garamond", 11.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.groupBox2.Location = new System.Drawing.Point(168, 328);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(144, 304);
			this.groupBox2.TabIndex = 2;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Manipulate";
			// 
			// btnSliceYRightDown
			// 
			this.btnSliceYRightDown.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceYRightDown.Location = new System.Drawing.Point(8, 168);
			this.btnSliceYRightDown.Name = "btnSliceYRightDown";
			this.btnSliceYRightDown.Size = new System.Drawing.Size(40, 32);
			this.btnSliceYRightDown.TabIndex = 6;
			this.btnSliceYRightDown.Text = "Down";
			this.btnSliceYRightDown.Click += new System.EventHandler(this.btnSliceYRightDown_Click);
			// 
			// btnSliceYLeftDown
			// 
			this.btnSliceYLeftDown.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceYLeftDown.Location = new System.Drawing.Point(8, 120);
			this.btnSliceYLeftDown.Name = "btnSliceYLeftDown";
			this.btnSliceYLeftDown.Size = new System.Drawing.Size(40, 32);
			this.btnSliceYLeftDown.TabIndex = 4;
			this.btnSliceYLeftDown.Text = "Down";
			this.btnSliceYLeftDown.Click += new System.EventHandler(this.btnSliceYLeftDown_Click);
			// 
			// btnSliceZBackCW
			// 
			this.btnSliceZBackCW.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceZBackCW.Location = new System.Drawing.Point(96, 264);
			this.btnSliceZBackCW.Name = "btnSliceZBackCW";
			this.btnSliceZBackCW.Size = new System.Drawing.Size(40, 32);
			this.btnSliceZBackCW.TabIndex = 11;
			this.btnSliceZBackCW.Text = "CW";
			this.btnSliceZBackCW.Click += new System.EventHandler(this.btnSliceZBackCW_Click);
			// 
			// btnSliceZBackCCW
			// 
			this.btnSliceZBackCCW.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceZBackCCW.Location = new System.Drawing.Point(8, 264);
			this.btnSliceZBackCCW.Name = "btnSliceZBackCCW";
			this.btnSliceZBackCCW.Size = new System.Drawing.Size(40, 32);
			this.btnSliceZBackCCW.TabIndex = 10;
			this.btnSliceZBackCCW.Text = "CCW";
			this.btnSliceZBackCCW.Click += new System.EventHandler(this.btnSliceZBackCCW_Click);
			// 
			// btnSliceZFrontCW
			// 
			this.btnSliceZFrontCW.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceZFrontCW.Location = new System.Drawing.Point(96, 216);
			this.btnSliceZFrontCW.Name = "btnSliceZFrontCW";
			this.btnSliceZFrontCW.Size = new System.Drawing.Size(40, 32);
			this.btnSliceZFrontCW.TabIndex = 9;
			this.btnSliceZFrontCW.Text = "CW";
			this.btnSliceZFrontCW.Click += new System.EventHandler(this.btnSliceZFrontCW_Click);
			// 
			// btnSliceZFrontCCW
			// 
			this.btnSliceZFrontCCW.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceZFrontCCW.Location = new System.Drawing.Point(8, 216);
			this.btnSliceZFrontCCW.Name = "btnSliceZFrontCCW";
			this.btnSliceZFrontCCW.Size = new System.Drawing.Size(40, 32);
			this.btnSliceZFrontCCW.TabIndex = 8;
			this.btnSliceZFrontCCW.Text = "CCW";
			this.btnSliceZFrontCCW.Click += new System.EventHandler(this.btnSliceZFrontCCW_Click);
			// 
			// btnSliceXBottomRight
			// 
			this.btnSliceXBottomRight.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceXBottomRight.Location = new System.Drawing.Point(96, 72);
			this.btnSliceXBottomRight.Name = "btnSliceXBottomRight";
			this.btnSliceXBottomRight.Size = new System.Drawing.Size(40, 32);
			this.btnSliceXBottomRight.TabIndex = 3;
			this.btnSliceXBottomRight.Text = "Right";
			this.btnSliceXBottomRight.Click += new System.EventHandler(this.btnSliceXBottomRight_Click);
			// 
			// btnSliceXBottomLeft
			// 
			this.btnSliceXBottomLeft.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceXBottomLeft.Location = new System.Drawing.Point(8, 72);
			this.btnSliceXBottomLeft.Name = "btnSliceXBottomLeft";
			this.btnSliceXBottomLeft.Size = new System.Drawing.Size(40, 32);
			this.btnSliceXBottomLeft.TabIndex = 2;
			this.btnSliceXBottomLeft.Text = "Left";
			this.btnSliceXBottomLeft.Click += new System.EventHandler(this.btnSliceXBottomLeft_Click);
			// 
			// btnSliceXTopRight
			// 
			this.btnSliceXTopRight.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceXTopRight.Location = new System.Drawing.Point(96, 24);
			this.btnSliceXTopRight.Name = "btnSliceXTopRight";
			this.btnSliceXTopRight.Size = new System.Drawing.Size(40, 32);
			this.btnSliceXTopRight.TabIndex = 1;
			this.btnSliceXTopRight.Text = "Right";
			this.btnSliceXTopRight.Click += new System.EventHandler(this.btnSliceXTopRight_Click);
			// 
			// btnSliceXTopLeft
			// 
			this.btnSliceXTopLeft.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceXTopLeft.Location = new System.Drawing.Point(8, 24);
			this.btnSliceXTopLeft.Name = "btnSliceXTopLeft";
			this.btnSliceXTopLeft.Size = new System.Drawing.Size(40, 32);
			this.btnSliceXTopLeft.TabIndex = 0;
			this.btnSliceXTopLeft.Text = "Left";
			this.btnSliceXTopLeft.Click += new System.EventHandler(this.btnSliceXTopLeft_Click);
			// 
			// btnSliceYLeftUp
			// 
			this.btnSliceYLeftUp.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceYLeftUp.Location = new System.Drawing.Point(96, 120);
			this.btnSliceYLeftUp.Name = "btnSliceYLeftUp";
			this.btnSliceYLeftUp.Size = new System.Drawing.Size(40, 32);
			this.btnSliceYLeftUp.TabIndex = 5;
			this.btnSliceYLeftUp.Text = "Up";
			this.btnSliceYLeftUp.Click += new System.EventHandler(this.btnSliceYLeftUp_Click);
			// 
			// btnSliceYRightUp
			// 
			this.btnSliceYRightUp.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSliceYRightUp.Location = new System.Drawing.Point(96, 168);
			this.btnSliceYRightUp.Name = "btnSliceYRightUp";
			this.btnSliceYRightUp.Size = new System.Drawing.Size(40, 32);
			this.btnSliceYRightUp.TabIndex = 7;
			this.btnSliceYRightUp.Text = "Up";
			this.btnSliceYRightUp.Click += new System.EventHandler(this.btnSliceYRightUp_Click);
			// 
			// btnRandom
			// 
			this.btnRandom.Font = new System.Drawing.Font("Garamond", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnRandom.Location = new System.Drawing.Point(8, 56);
			this.btnRandom.Name = "btnRandom";
			this.btnRandom.Size = new System.Drawing.Size(88, 23);
			this.btnRandom.TabIndex = 1;
			this.btnRandom.Text = "Randomize";
			this.btnRandom.Click += new System.EventHandler(this.btnRandom_Click);
			// 
			// btnReset
			// 
			this.btnReset.Font = new System.Drawing.Font("Garamond", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnReset.Location = new System.Drawing.Point(8, 24);
			this.btnReset.Name = "btnReset";
			this.btnReset.Size = new System.Drawing.Size(88, 23);
			this.btnReset.TabIndex = 0;
			this.btnReset.Text = "Reset Cube";
			this.btnReset.Click += new System.EventHandler(this.btnReset_Click);
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.Add(this.groupBox5);
			this.groupBox3.Controls.Add(this.txtStatus);
			this.groupBox3.Controls.Add(this.btnSolve);
			this.groupBox3.Controls.Add(this.lstSolution);
			this.groupBox3.Controls.Add(this.btnReset);
			this.groupBox3.Controls.Add(this.btnRandom);
			this.groupBox3.Font = new System.Drawing.Font("Garamond", 11.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.groupBox3.Location = new System.Drawing.Point(328, 328);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(304, 304);
			this.groupBox3.TabIndex = 4;
			this.groupBox3.TabStop = false;
			this.groupBox3.Text = "Solution";
			// 
			// groupBox5
			// 
			this.groupBox5.Controls.Add(this.label3);
			this.groupBox5.Controls.Add(this.label2);
			this.groupBox5.Controls.Add(this.trackThreshold);
			this.groupBox5.Location = new System.Drawing.Point(104, 24);
			this.groupBox5.Name = "groupBox5";
			this.groupBox5.Size = new System.Drawing.Size(192, 88);
			this.groupBox5.TabIndex = 4;
			this.groupBox5.TabStop = false;
			this.groupBox5.Text = "Search Method";
			// 
			// label3
			// 
			this.label3.Font = new System.Drawing.Font("Garamond", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label3.Location = new System.Drawing.Point(144, 24);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(40, 23);
			this.label3.TabIndex = 6;
			this.label3.Text = "Full";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label2
			// 
			this.label2.Font = new System.Drawing.Font("Garamond", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label2.Location = new System.Drawing.Point(16, 24);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(40, 23);
			this.label2.TabIndex = 5;
			this.label2.Text = "Quick";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// trackThreshold
			// 
			this.trackThreshold.AutoSize = false;
			this.trackThreshold.Location = new System.Drawing.Point(8, 48);
			this.trackThreshold.Maximum = 100;
			this.trackThreshold.Minimum = 1;
			this.trackThreshold.Name = "trackThreshold";
			this.trackThreshold.Size = new System.Drawing.Size(176, 32);
			this.trackThreshold.TabIndex = 0;
			this.trackThreshold.TickFrequency = 5;
			this.trackThreshold.Value = 100;
			this.trackThreshold.Scroll += new System.EventHandler(this.trackThreshold_Scroll);
			// 
			// txtStatus
			// 
			this.txtStatus.Font = new System.Drawing.Font("Garamond", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.txtStatus.Location = new System.Drawing.Point(8, 120);
			this.txtStatus.Name = "txtStatus";
			this.txtStatus.ReadOnly = true;
			this.txtStatus.Size = new System.Drawing.Size(288, 21);
			this.txtStatus.TabIndex = 3;
			this.txtStatus.Text = "Status: Ready.";
			// 
			// btnSolve
			// 
			this.btnSolve.Font = new System.Drawing.Font("Garamond", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.btnSolve.Location = new System.Drawing.Point(8, 88);
			this.btnSolve.Name = "btnSolve";
			this.btnSolve.Size = new System.Drawing.Size(88, 23);
			this.btnSolve.TabIndex = 2;
			this.btnSolve.Text = "Find Solution";
			this.btnSolve.Click += new System.EventHandler(this.btnSolve_Click);
			// 
			// lstSolution
			// 
			this.lstSolution.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																												  this.StepColumn,
																												  this.ActionColumn,
																												  this.ScoreColumn});
			this.lstSolution.FullRowSelect = true;
			this.lstSolution.Location = new System.Drawing.Point(8, 152);
			this.lstSolution.MultiSelect = false;
			this.lstSolution.Name = "lstSolution";
			this.lstSolution.Size = new System.Drawing.Size(288, 144);
			this.lstSolution.TabIndex = 3;
			this.lstSolution.View = System.Windows.Forms.View.Details;
			this.lstSolution.SelectedIndexChanged += new System.EventHandler(this.lstSolution_SelectedIndexChanged);
			// 
			// StepColumn
			// 
			this.StepColumn.Text = "Step";
			// 
			// ActionColumn
			// 
			this.ActionColumn.Text = "Action";
			this.ActionColumn.Width = 136;
			// 
			// ScoreColumn
			// 
			this.ScoreColumn.Text = "Score";
			// 
			// groupBox4
			// 
			this.groupBox4.Controls.Add(this.label61);
			this.groupBox4.Controls.Add(this.label60);
			this.groupBox4.Controls.Add(this.label59);
			this.groupBox4.Controls.Add(this.label58);
			this.groupBox4.Controls.Add(this.label57);
			this.groupBox4.Controls.Add(this.label56);
			this.groupBox4.Controls.Add(this.txtFace69);
			this.groupBox4.Controls.Add(this.txtFace68);
			this.groupBox4.Controls.Add(this.txtFace67);
			this.groupBox4.Controls.Add(this.txtFace66);
			this.groupBox4.Controls.Add(this.txtFace65);
			this.groupBox4.Controls.Add(this.txtFace64);
			this.groupBox4.Controls.Add(this.txtFace63);
			this.groupBox4.Controls.Add(this.txtFace62);
			this.groupBox4.Controls.Add(this.txtFace61);
			this.groupBox4.Controls.Add(this.txtFace59);
			this.groupBox4.Controls.Add(this.txtFace58);
			this.groupBox4.Controls.Add(this.txtFace57);
			this.groupBox4.Controls.Add(this.txtFace56);
			this.groupBox4.Controls.Add(this.txtFace55);
			this.groupBox4.Controls.Add(this.txtFace54);
			this.groupBox4.Controls.Add(this.txtFace53);
			this.groupBox4.Controls.Add(this.txtFace52);
			this.groupBox4.Controls.Add(this.txtFace51);
			this.groupBox4.Controls.Add(this.txtFace49);
			this.groupBox4.Controls.Add(this.txtFace48);
			this.groupBox4.Controls.Add(this.txtFace47);
			this.groupBox4.Controls.Add(this.txtFace46);
			this.groupBox4.Controls.Add(this.txtFace45);
			this.groupBox4.Controls.Add(this.txtFace44);
			this.groupBox4.Controls.Add(this.txtFace43);
			this.groupBox4.Controls.Add(this.txtFace42);
			this.groupBox4.Controls.Add(this.txtFace41);
			this.groupBox4.Controls.Add(this.txtFace39);
			this.groupBox4.Controls.Add(this.txtFace38);
			this.groupBox4.Controls.Add(this.txtFace37);
			this.groupBox4.Controls.Add(this.txtFace36);
			this.groupBox4.Controls.Add(this.txtFace35);
			this.groupBox4.Controls.Add(this.txtFace34);
			this.groupBox4.Controls.Add(this.txtFace33);
			this.groupBox4.Controls.Add(this.txtFace32);
			this.groupBox4.Controls.Add(this.txtFace31);
			this.groupBox4.Controls.Add(this.txtFace29);
			this.groupBox4.Controls.Add(this.txtFace28);
			this.groupBox4.Controls.Add(this.txtFace27);
			this.groupBox4.Controls.Add(this.txtFace26);
			this.groupBox4.Controls.Add(this.txtFace25);
			this.groupBox4.Controls.Add(this.txtFace24);
			this.groupBox4.Controls.Add(this.txtFace23);
			this.groupBox4.Controls.Add(this.txtFace22);
			this.groupBox4.Controls.Add(this.txtFace21);
			this.groupBox4.Controls.Add(this.txtFace19);
			this.groupBox4.Controls.Add(this.txtFace18);
			this.groupBox4.Controls.Add(this.txtFace17);
			this.groupBox4.Controls.Add(this.txtFace16);
			this.groupBox4.Controls.Add(this.txtFace15);
			this.groupBox4.Controls.Add(this.txtFace14);
			this.groupBox4.Controls.Add(this.txtFace13);
			this.groupBox4.Controls.Add(this.txtFace12);
			this.groupBox4.Controls.Add(this.txtFace11);
			this.groupBox4.Font = new System.Drawing.Font("Garamond", 11.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.groupBox4.Location = new System.Drawing.Point(328, 8);
			this.groupBox4.Name = "groupBox4";
			this.groupBox4.Size = new System.Drawing.Size(304, 304);
			this.groupBox4.TabIndex = 5;
			this.groupBox4.TabStop = false;
			this.groupBox4.Text = "Configuration";
			// 
			// label61
			// 
			this.label61.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label61.Location = new System.Drawing.Point(216, 176);
			this.label61.Name = "label61";
			this.label61.Size = new System.Drawing.Size(72, 23);
			this.label61.TabIndex = 59;
			this.label61.Text = "Face 6";
			this.label61.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label60
			// 
			this.label60.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label60.Location = new System.Drawing.Point(120, 176);
			this.label60.Name = "label60";
			this.label60.Size = new System.Drawing.Size(72, 23);
			this.label60.TabIndex = 58;
			this.label60.Text = "Face 5";
			this.label60.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label59
			// 
			this.label59.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label59.Location = new System.Drawing.Point(16, 176);
			this.label59.Name = "label59";
			this.label59.Size = new System.Drawing.Size(72, 23);
			this.label59.TabIndex = 57;
			this.label59.Text = "Face 4";
			this.label59.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label58
			// 
			this.label58.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label58.Location = new System.Drawing.Point(216, 48);
			this.label58.Name = "label58";
			this.label58.Size = new System.Drawing.Size(72, 23);
			this.label58.TabIndex = 56;
			this.label58.Text = "Face 3";
			this.label58.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label57
			// 
			this.label57.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label57.Location = new System.Drawing.Point(120, 48);
			this.label57.Name = "label57";
			this.label57.Size = new System.Drawing.Size(72, 23);
			this.label57.TabIndex = 55;
			this.label57.Text = "Face 2";
			this.label57.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label56
			// 
			this.label56.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label56.Location = new System.Drawing.Point(16, 48);
			this.label56.Name = "label56";
			this.label56.Size = new System.Drawing.Size(72, 23);
			this.label56.TabIndex = 54;
			this.label56.Text = "Face 1";
			this.label56.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace69
			// 
			this.txtFace69.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace69.Location = new System.Drawing.Point(265, 256);
			this.txtFace69.Name = "txtFace69";
			this.txtFace69.Size = new System.Drawing.Size(23, 23);
			this.txtFace69.TabIndex = 53;
			this.txtFace69.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace68
			// 
			this.txtFace68.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace68.Location = new System.Drawing.Point(241, 256);
			this.txtFace68.Name = "txtFace68";
			this.txtFace68.Size = new System.Drawing.Size(23, 23);
			this.txtFace68.TabIndex = 52;
			this.txtFace68.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace67
			// 
			this.txtFace67.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace67.Location = new System.Drawing.Point(217, 256);
			this.txtFace67.Name = "txtFace67";
			this.txtFace67.Size = new System.Drawing.Size(23, 23);
			this.txtFace67.TabIndex = 51;
			this.txtFace67.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace66
			// 
			this.txtFace66.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace66.Location = new System.Drawing.Point(265, 232);
			this.txtFace66.Name = "txtFace66";
			this.txtFace66.Size = new System.Drawing.Size(23, 23);
			this.txtFace66.TabIndex = 50;
			this.txtFace66.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace65
			// 
			this.txtFace65.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace65.Location = new System.Drawing.Point(241, 232);
			this.txtFace65.Name = "txtFace65";
			this.txtFace65.Size = new System.Drawing.Size(23, 23);
			this.txtFace65.TabIndex = 49;
			this.txtFace65.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace64
			// 
			this.txtFace64.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace64.Location = new System.Drawing.Point(217, 232);
			this.txtFace64.Name = "txtFace64";
			this.txtFace64.Size = new System.Drawing.Size(23, 23);
			this.txtFace64.TabIndex = 48;
			this.txtFace64.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace63
			// 
			this.txtFace63.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace63.Location = new System.Drawing.Point(265, 208);
			this.txtFace63.Name = "txtFace63";
			this.txtFace63.Size = new System.Drawing.Size(23, 23);
			this.txtFace63.TabIndex = 47;
			this.txtFace63.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace62
			// 
			this.txtFace62.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace62.Location = new System.Drawing.Point(241, 208);
			this.txtFace62.Name = "txtFace62";
			this.txtFace62.Size = new System.Drawing.Size(23, 23);
			this.txtFace62.TabIndex = 46;
			this.txtFace62.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace61
			// 
			this.txtFace61.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace61.Location = new System.Drawing.Point(217, 208);
			this.txtFace61.Name = "txtFace61";
			this.txtFace61.Size = new System.Drawing.Size(23, 23);
			this.txtFace61.TabIndex = 45;
			this.txtFace61.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace59
			// 
			this.txtFace59.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace59.Location = new System.Drawing.Point(168, 256);
			this.txtFace59.Name = "txtFace59";
			this.txtFace59.Size = new System.Drawing.Size(23, 23);
			this.txtFace59.TabIndex = 44;
			this.txtFace59.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace58
			// 
			this.txtFace58.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace58.Location = new System.Drawing.Point(144, 256);
			this.txtFace58.Name = "txtFace58";
			this.txtFace58.Size = new System.Drawing.Size(23, 23);
			this.txtFace58.TabIndex = 43;
			this.txtFace58.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace57
			// 
			this.txtFace57.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace57.Location = new System.Drawing.Point(120, 256);
			this.txtFace57.Name = "txtFace57";
			this.txtFace57.Size = new System.Drawing.Size(23, 23);
			this.txtFace57.TabIndex = 42;
			this.txtFace57.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace56
			// 
			this.txtFace56.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace56.Location = new System.Drawing.Point(168, 232);
			this.txtFace56.Name = "txtFace56";
			this.txtFace56.Size = new System.Drawing.Size(23, 23);
			this.txtFace56.TabIndex = 41;
			this.txtFace56.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace55
			// 
			this.txtFace55.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace55.Location = new System.Drawing.Point(144, 232);
			this.txtFace55.Name = "txtFace55";
			this.txtFace55.Size = new System.Drawing.Size(23, 23);
			this.txtFace55.TabIndex = 40;
			this.txtFace55.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace54
			// 
			this.txtFace54.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace54.Location = new System.Drawing.Point(120, 232);
			this.txtFace54.Name = "txtFace54";
			this.txtFace54.Size = new System.Drawing.Size(23, 23);
			this.txtFace54.TabIndex = 39;
			this.txtFace54.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace53
			// 
			this.txtFace53.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace53.Location = new System.Drawing.Point(168, 208);
			this.txtFace53.Name = "txtFace53";
			this.txtFace53.Size = new System.Drawing.Size(23, 23);
			this.txtFace53.TabIndex = 38;
			this.txtFace53.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace52
			// 
			this.txtFace52.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace52.Location = new System.Drawing.Point(144, 208);
			this.txtFace52.Name = "txtFace52";
			this.txtFace52.Size = new System.Drawing.Size(23, 23);
			this.txtFace52.TabIndex = 37;
			this.txtFace52.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace51
			// 
			this.txtFace51.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace51.Location = new System.Drawing.Point(120, 208);
			this.txtFace51.Name = "txtFace51";
			this.txtFace51.Size = new System.Drawing.Size(23, 23);
			this.txtFace51.TabIndex = 36;
			this.txtFace51.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace49
			// 
			this.txtFace49.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace49.Location = new System.Drawing.Point(65, 256);
			this.txtFace49.Name = "txtFace49";
			this.txtFace49.Size = new System.Drawing.Size(23, 23);
			this.txtFace49.TabIndex = 35;
			this.txtFace49.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace48
			// 
			this.txtFace48.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace48.Location = new System.Drawing.Point(41, 256);
			this.txtFace48.Name = "txtFace48";
			this.txtFace48.Size = new System.Drawing.Size(23, 23);
			this.txtFace48.TabIndex = 34;
			this.txtFace48.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace47
			// 
			this.txtFace47.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace47.Location = new System.Drawing.Point(17, 256);
			this.txtFace47.Name = "txtFace47";
			this.txtFace47.Size = new System.Drawing.Size(23, 23);
			this.txtFace47.TabIndex = 33;
			this.txtFace47.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace46
			// 
			this.txtFace46.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace46.Location = new System.Drawing.Point(65, 232);
			this.txtFace46.Name = "txtFace46";
			this.txtFace46.Size = new System.Drawing.Size(23, 23);
			this.txtFace46.TabIndex = 32;
			this.txtFace46.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace45
			// 
			this.txtFace45.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace45.Location = new System.Drawing.Point(41, 232);
			this.txtFace45.Name = "txtFace45";
			this.txtFace45.Size = new System.Drawing.Size(23, 23);
			this.txtFace45.TabIndex = 31;
			this.txtFace45.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace44
			// 
			this.txtFace44.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace44.Location = new System.Drawing.Point(17, 232);
			this.txtFace44.Name = "txtFace44";
			this.txtFace44.Size = new System.Drawing.Size(23, 23);
			this.txtFace44.TabIndex = 30;
			this.txtFace44.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace43
			// 
			this.txtFace43.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace43.Location = new System.Drawing.Point(65, 208);
			this.txtFace43.Name = "txtFace43";
			this.txtFace43.Size = new System.Drawing.Size(23, 23);
			this.txtFace43.TabIndex = 29;
			this.txtFace43.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace42
			// 
			this.txtFace42.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace42.Location = new System.Drawing.Point(41, 208);
			this.txtFace42.Name = "txtFace42";
			this.txtFace42.Size = new System.Drawing.Size(23, 23);
			this.txtFace42.TabIndex = 28;
			this.txtFace42.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace41
			// 
			this.txtFace41.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace41.Location = new System.Drawing.Point(17, 208);
			this.txtFace41.Name = "txtFace41";
			this.txtFace41.Size = new System.Drawing.Size(23, 23);
			this.txtFace41.TabIndex = 27;
			this.txtFace41.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace39
			// 
			this.txtFace39.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace39.Location = new System.Drawing.Point(264, 128);
			this.txtFace39.Name = "txtFace39";
			this.txtFace39.Size = new System.Drawing.Size(23, 23);
			this.txtFace39.TabIndex = 26;
			this.txtFace39.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace38
			// 
			this.txtFace38.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace38.Location = new System.Drawing.Point(240, 128);
			this.txtFace38.Name = "txtFace38";
			this.txtFace38.Size = new System.Drawing.Size(23, 23);
			this.txtFace38.TabIndex = 25;
			this.txtFace38.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace37
			// 
			this.txtFace37.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace37.Location = new System.Drawing.Point(216, 128);
			this.txtFace37.Name = "txtFace37";
			this.txtFace37.Size = new System.Drawing.Size(23, 23);
			this.txtFace37.TabIndex = 24;
			this.txtFace37.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace36
			// 
			this.txtFace36.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace36.Location = new System.Drawing.Point(264, 104);
			this.txtFace36.Name = "txtFace36";
			this.txtFace36.Size = new System.Drawing.Size(23, 23);
			this.txtFace36.TabIndex = 23;
			this.txtFace36.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace35
			// 
			this.txtFace35.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace35.Location = new System.Drawing.Point(240, 104);
			this.txtFace35.Name = "txtFace35";
			this.txtFace35.Size = new System.Drawing.Size(23, 23);
			this.txtFace35.TabIndex = 22;
			this.txtFace35.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace34
			// 
			this.txtFace34.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace34.Location = new System.Drawing.Point(216, 104);
			this.txtFace34.Name = "txtFace34";
			this.txtFace34.Size = new System.Drawing.Size(23, 23);
			this.txtFace34.TabIndex = 21;
			this.txtFace34.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace33
			// 
			this.txtFace33.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace33.Location = new System.Drawing.Point(264, 80);
			this.txtFace33.Name = "txtFace33";
			this.txtFace33.Size = new System.Drawing.Size(23, 23);
			this.txtFace33.TabIndex = 20;
			this.txtFace33.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace32
			// 
			this.txtFace32.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace32.Location = new System.Drawing.Point(240, 80);
			this.txtFace32.Name = "txtFace32";
			this.txtFace32.Size = new System.Drawing.Size(23, 23);
			this.txtFace32.TabIndex = 19;
			this.txtFace32.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace31
			// 
			this.txtFace31.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace31.Location = new System.Drawing.Point(216, 80);
			this.txtFace31.Name = "txtFace31";
			this.txtFace31.Size = new System.Drawing.Size(23, 23);
			this.txtFace31.TabIndex = 18;
			this.txtFace31.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace29
			// 
			this.txtFace29.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace29.Location = new System.Drawing.Point(168, 128);
			this.txtFace29.Name = "txtFace29";
			this.txtFace29.Size = new System.Drawing.Size(23, 23);
			this.txtFace29.TabIndex = 17;
			this.txtFace29.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace28
			// 
			this.txtFace28.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace28.Location = new System.Drawing.Point(144, 128);
			this.txtFace28.Name = "txtFace28";
			this.txtFace28.Size = new System.Drawing.Size(23, 23);
			this.txtFace28.TabIndex = 16;
			this.txtFace28.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace27
			// 
			this.txtFace27.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace27.Location = new System.Drawing.Point(120, 128);
			this.txtFace27.Name = "txtFace27";
			this.txtFace27.Size = new System.Drawing.Size(23, 23);
			this.txtFace27.TabIndex = 15;
			this.txtFace27.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace26
			// 
			this.txtFace26.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace26.Location = new System.Drawing.Point(168, 104);
			this.txtFace26.Name = "txtFace26";
			this.txtFace26.Size = new System.Drawing.Size(23, 23);
			this.txtFace26.TabIndex = 14;
			this.txtFace26.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace25
			// 
			this.txtFace25.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace25.Location = new System.Drawing.Point(144, 104);
			this.txtFace25.Name = "txtFace25";
			this.txtFace25.Size = new System.Drawing.Size(23, 23);
			this.txtFace25.TabIndex = 13;
			this.txtFace25.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace24
			// 
			this.txtFace24.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace24.Location = new System.Drawing.Point(120, 104);
			this.txtFace24.Name = "txtFace24";
			this.txtFace24.Size = new System.Drawing.Size(23, 23);
			this.txtFace24.TabIndex = 12;
			this.txtFace24.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace23
			// 
			this.txtFace23.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace23.Location = new System.Drawing.Point(168, 80);
			this.txtFace23.Name = "txtFace23";
			this.txtFace23.Size = new System.Drawing.Size(23, 23);
			this.txtFace23.TabIndex = 11;
			this.txtFace23.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace22
			// 
			this.txtFace22.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace22.Location = new System.Drawing.Point(144, 80);
			this.txtFace22.Name = "txtFace22";
			this.txtFace22.Size = new System.Drawing.Size(23, 23);
			this.txtFace22.TabIndex = 10;
			this.txtFace22.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace21
			// 
			this.txtFace21.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace21.Location = new System.Drawing.Point(120, 80);
			this.txtFace21.Name = "txtFace21";
			this.txtFace21.Size = new System.Drawing.Size(23, 23);
			this.txtFace21.TabIndex = 9;
			this.txtFace21.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace19
			// 
			this.txtFace19.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace19.Location = new System.Drawing.Point(64, 128);
			this.txtFace19.Name = "txtFace19";
			this.txtFace19.Size = new System.Drawing.Size(23, 23);
			this.txtFace19.TabIndex = 8;
			this.txtFace19.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace18
			// 
			this.txtFace18.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace18.Location = new System.Drawing.Point(40, 128);
			this.txtFace18.Name = "txtFace18";
			this.txtFace18.Size = new System.Drawing.Size(23, 23);
			this.txtFace18.TabIndex = 7;
			this.txtFace18.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace17
			// 
			this.txtFace17.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace17.Location = new System.Drawing.Point(16, 128);
			this.txtFace17.Name = "txtFace17";
			this.txtFace17.Size = new System.Drawing.Size(23, 23);
			this.txtFace17.TabIndex = 6;
			this.txtFace17.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace16
			// 
			this.txtFace16.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace16.Location = new System.Drawing.Point(64, 104);
			this.txtFace16.Name = "txtFace16";
			this.txtFace16.Size = new System.Drawing.Size(23, 23);
			this.txtFace16.TabIndex = 5;
			this.txtFace16.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace15
			// 
			this.txtFace15.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace15.Location = new System.Drawing.Point(40, 104);
			this.txtFace15.Name = "txtFace15";
			this.txtFace15.Size = new System.Drawing.Size(23, 23);
			this.txtFace15.TabIndex = 4;
			this.txtFace15.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace14
			// 
			this.txtFace14.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace14.Location = new System.Drawing.Point(16, 104);
			this.txtFace14.Name = "txtFace14";
			this.txtFace14.Size = new System.Drawing.Size(23, 23);
			this.txtFace14.TabIndex = 3;
			this.txtFace14.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace13
			// 
			this.txtFace13.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace13.Location = new System.Drawing.Point(64, 80);
			this.txtFace13.Name = "txtFace13";
			this.txtFace13.Size = new System.Drawing.Size(23, 23);
			this.txtFace13.TabIndex = 2;
			this.txtFace13.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace12
			// 
			this.txtFace12.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace12.Location = new System.Drawing.Point(40, 80);
			this.txtFace12.Name = "txtFace12";
			this.txtFace12.Size = new System.Drawing.Size(23, 23);
			this.txtFace12.TabIndex = 1;
			this.txtFace12.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtFace11
			// 
			this.txtFace11.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.txtFace11.Location = new System.Drawing.Point(16, 80);
			this.txtFace11.Name = "txtFace11";
			this.txtFace11.Size = new System.Drawing.Size(23, 23);
			this.txtFace11.TabIndex = 0;
			this.txtFace11.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// timerUpdate
			// 
			this.timerUpdate.Enabled = true;
			this.timerUpdate.Tick += new System.EventHandler(this.timerUpdate_Tick);
			// 
			// menuMain
			// 
			this.menuMain.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																											this.menuItem1});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																											 this.menuHelpAbout,
																											 this.menuExit});
			this.menuItem1.Text = "CubeSearch";
			// 
			// menuHelpAbout
			// 
			this.menuHelpAbout.Index = 0;
			this.menuHelpAbout.Text = "Help/About";
			this.menuHelpAbout.Click += new System.EventHandler(this.menuHelpAbout_Click);
			// 
			// menuExit
			// 
			this.menuExit.Index = 1;
			this.menuExit.Text = "Exit";
			this.menuExit.Click += new System.EventHandler(this.menuExit_Click);
			// 
			// CubeSearch
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(638, 636);
			this.Controls.Add(this.groupBox3);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.picCube);
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.groupBox4);
			this.Font = new System.Drawing.Font("Garamond", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.KeyPreview = true;
			this.MaximizeBox = false;
			this.Menu = this.menuMain;
			this.MinimizeBox = false;
			this.Name = "CubeSearch";
			this.Text = "CubeSearch - Rubik\'s Cube Heuristic Solution";
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.CubeSearch_KeyDown);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.CubeSearch_Closing);
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.groupBox5.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.trackThreshold)).EndInit();
			this.groupBox4.ResumeLayout(false);
			this.ResumeLayout(false);

		}

		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}
		#endregion

		[STAThread]
		static void Main() {
			Application.Run(new CubeSearch());
		}

		#endregion

		#region Button-click event handlers

		private void btnReset_Click(object sender, System.EventArgs e) {
			cube.Reset();
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void CubeSearch_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e) {     // process keypresses
			if (e.KeyCode == Keys.W) {
				cube.RotateView(Axis.Y, Direction.Up);
				picCube.Refresh();
			}
			else if (e.KeyCode == Keys.S) {
				cube.RotateView(Axis.Y, Direction.Down);
				picCube.Refresh();
			}
			else if (e.KeyCode == Keys.A) {
				cube.RotateView(Axis.X, Direction.Left);
				picCube.Refresh();
			}
			else if (e.KeyCode == Keys.D) {
				cube.RotateView(Axis.X, Direction.Right);
				picCube.Refresh();
			}
			else if (e.KeyCode == Keys.Q) {
				cube.RotateView(Axis.Z, Direction.CCW);
				picCube.Refresh();
			}
			else if (e.KeyCode == Keys.E) {
				cube.RotateView(Axis.Z, Direction.CW);
				picCube.Refresh();
			}
		}

		private void menuHelpAbout_Click(object sender, System.EventArgs e) {
			(new CubeSearchHelp()).ShowDialog();
		}

		private void menuExit_Click(object sender, System.EventArgs e) {
			Close();
		}

		// cube and view manipulation functions

		private void btnRotateXLeft_Click(object sender, System.EventArgs e) {
			cube.RotateView(Axis.X, Direction.Left);
			picCube.Refresh();
		}

		private void btnRotateXRight_Click(object sender, System.EventArgs e) {
			cube.RotateView(Axis.X, Direction.Right);
			picCube.Refresh();
		}

		private void btnRotateYUp_Click(object sender, System.EventArgs e) {
			cube.RotateView(Axis.Y, Direction.Up);
			picCube.Refresh();
		}

		private void btnRotateYDown_Click(object sender, System.EventArgs e) {
			cube.RotateView(Axis.Y, Direction.Down);
			picCube.Refresh();
		}

		private void btnRotateZCCCW_Click(object sender, System.EventArgs e) {
			cube.RotateView(Axis.Z, Direction.CCW);
			picCube.Refresh();
		}

		private void btnRotateZCW_Click(object sender, System.EventArgs e) {
			cube.RotateView(Axis.Z, Direction.CW);
			picCube.Refresh();
		}

		private void btnSliceXTopLeft_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Left]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceXTopRight_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Right]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceXBottomLeft_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Left]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceXBottomRight_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Right]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceYLeftUp_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Up]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceYLeftDown_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Down]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceYRightUp_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Up]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceYRightDown_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Down]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceZFrontCCW_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CCW]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceZFrontCW_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CW]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceZBackCCW_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CCW]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnSliceZBackCW_Click(object sender, System.EventArgs e) {
			Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CW]);
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void btnRandom_Click(object sender, System.EventArgs e) {     // randomize the cube
			Random r = new Random((int) System.DateTime.Now.Ticks);
			cube.Reset();
			for (int i = 0; i < 10; i++) {  // apply 10 random transitions
				int a = r.Next(3);
				int s = r.Next(2);
				int d = r.Next(2);
				if (a == 0)
					Cube.Transform(cube.iFace, new Transform(Axis.X, s == 0 ? Slice.Top : Slice.Bottom, d == 0 ? Direction.Left : Direction.Right)); 
				else if (a == 1)
					Cube.Transform(cube.iFace, new Transform(Axis.Y, s == 0 ? Slice.Left : Slice.Right, d == 0 ? Direction.Up : Direction.Down)); 
				else if (a == 2)
					Cube.Transform(cube.iFace, new Transform(Axis.Z, s == 0 ? Slice.Front : Slice.Back, d == 0 ? Direction.CCW : Direction.CW)); 
			}
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void lstSolution_SelectedIndexChanged(object sender, System.EventArgs e) {     // move to a particular state in the solution list
			if ((lstSolution.SelectedItems.Count == 1) && (cube.solution != null)) {
				int iStepNumber = Int32.Parse(lstSolution.SelectedItems[0].SubItems[0].Text.ToString());
				SolutionStep s = cube.solution;
				while ((s.iStepNumber != iStepNumber) && (s.parent != null))
					s = s.parent;
				for (int i = 0; i < 6; i++) {
					for (int j = 0; j < 9; j++) {
						cube.iFace[i,j] = s.iFace[i,j];
					}
				}
				picCube.Refresh();
				UpdateConfiguration();
			}
		}

		private void lstColorSchemes_SelectedIndexChanged(object sender, System.EventArgs e) {     // selecting new color scheme from list
			if (lstColorSchemes.SelectedItems.Count == 1) {
				if (lstColorSchemes.SelectedItem.ToString() == "Original") {
					colors[0] = Color.Red;
					colors[1] = Color.Lime;
					colors[2] = Color.Orange;
					colors[3] = Color.Blue;
					colors[4] = Color.Yellow;
					colors[5] = Color.White;
				}
				else if (lstColorSchemes.SelectedItem.ToString() == "Greyscale") {
					colors[0] = Color.White;
					colors[1] = Color.FromArgb(50, 50, 50);
					colors[2] = Color.FromArgb(86, 86, 86);
					colors[3] = Color.FromArgb(128, 128, 128);
					colors[4] = Color.FromArgb(192, 192, 192);
					colors[5] = Color.FromArgb(226, 226, 226);
				}
				else if (lstColorSchemes.SelectedItem.ToString() == "Pastels") {
					colors[0] = Color.LightGreen;
					colors[1] = Color.LightBlue;
					colors[2] = Color.LightGoldenrodYellow;
					colors[3] = Color.LightSalmon;
					colors[4] = Color.LightPink;
					colors[5] = Color.LightCyan;
				}
				else if (lstColorSchemes.SelectedItem.ToString() == "Blues") {
					colors[0] = Color.Blue;
					colors[1] = Color.LightBlue;
					colors[2] = Color.MidnightBlue;
					colors[3] = Color.DarkSlateBlue;
					colors[4] = Color.Cyan;
					colors[5] = Color.DarkTurquoise;
				}
				else if (lstColorSchemes.SelectedItem.ToString() == "Reds") {
					colors[0] = Color.Red;
					colors[1] = Color.Maroon;
					colors[2] = Color.Tomato;
					colors[3] = Color.LightPink;
					colors[4] = Color.DarkSalmon;
					colors[5] = Color.Crimson;
				}
				else if (lstColorSchemes.SelectedItem.ToString() == "Greens") {
					colors[0] = Color.ForestGreen;
					colors[1] = Color.LightGreen;
					colors[2] = Color.SeaGreen;
					colors[3] = Color.DarkOliveGreen;
					colors[4] = Color.SpringGreen;
					colors[5] = Color.LawnGreen;
				}
				else if (lstColorSchemes.SelectedItem.ToString() == "Earth Tones") {
					colors[0] = Color.Brown;
					colors[1] = Color.Khaki;
					colors[2] = Color.ForestGreen;
					colors[3] = Color.BlanchedAlmond;
					colors[4] = Color.BurlyWood;
					colors[5] = Color.LawnGreen;
				}
				else if (lstColorSchemes.SelectedItem.ToString() == "Neon") {
					colors[0] = Color.Magenta;
					colors[1] = Color.Lime;
					colors[2] = Color.Yellow;
					colors[3] = Color.Aqua;
					colors[4] = Color.Gold;
					colors[5] = Color.DodgerBlue;
				}
			}
			picCube.Refresh();
			UpdateConfiguration();
		}

		private void EnableButtons(bool enable) {     // turn all buttons on or off (used when solution-finder thread starts or stops)
			this.btnSliceXTopLeft.Enabled =
			this.btnSliceXTopRight.Enabled =
			this.btnSliceXBottomLeft.Enabled =
			this.btnSliceXBottomRight.Enabled =
			this.btnSliceYLeftDown.Enabled =
			this.btnSliceYLeftUp.Enabled =
			this.btnSliceYRightDown.Enabled =
			this.btnSliceYRightUp.Enabled =
			this.btnSliceZFrontCCW.Enabled =
			this.btnSliceZFrontCW.Enabled =
			this.btnSliceZBackCCW.Enabled =
			this.btnSliceZBackCW.Enabled =
			this.btnRandom.Enabled =
			this.btnReset.Enabled = enable;
		}

		private void picCube_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
			iMouseX = e.X;
			iMouseY = e.Y;
			if (bMouseDown == true) {
				switch (iMouseSpot) {
					case 1:
						if (iRotated == 0) {
							if (iMouseX < 70) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Left]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseX > 114) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Right]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseX > 70)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Right]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseX < 114)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Left]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 2:
						if (iRotated == 0) {
							if (iMouseY < 160) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Up]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseY > 204) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Down]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseY > 160)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Down]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseY < 204)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Up]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 3:
						if (iRotated == 0) {
							if (iMouseY < 160) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Up]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseY > 204) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Down]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseY > 160)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Down]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseY < 204)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Up]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 4:
						if (iRotated == 0) {
							if (iMouseX < 70) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Left]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseX > 114) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Right]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseX > 70)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Right]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseX < 114)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Left]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 5:
						if (iRotated == 0) {
							if (iMouseX < 70) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CCW]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseX > 147) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CW]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseX > 70)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CW]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseX < 147)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CCW]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 6:
						if (iRotated == 0) {
							if (iMouseX < 136) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CCW]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseX > 214) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CW]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseX > 136)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CW]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseX < 214)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CCW]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 7:
						if (iRotated == 0) {
							if (iMouseY < 72) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Up]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseY > 94) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Down]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseY > 72)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Down]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseY < 94)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Left, (int) Direction.Up]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 8:
						if (iRotated == 0) {
							if (iMouseY < 72) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Up]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseY > 94) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Down]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseY > 72)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Down]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseY < 94)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Y, (int) Slice.Right, (int) Direction.Up]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 9:
						if (iRotated == 0) {
							if (iMouseX < 192) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Left]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseX > 225) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Right]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseX > 192)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Right]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseX < 225)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Top, (int) Direction.Left]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 10:
						if (iRotated == 0) {
							if (iMouseY < 138) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CCW]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseY > 204) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CW]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseY > 138)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CW]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseY < 204)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Front, (int) Direction.CCW]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 11:
						if (iRotated == 0) {
							if (iMouseY < 94) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CCW]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseY > 160) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CW]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseY > 94)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CW]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseY < 160)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.Z, (int) Slice.Back, (int) Direction.CCW]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					case 12:
						if (iRotated == 0) {
							if (iMouseX < 192) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Left]);
								iRotated = -1;
								UpdateConfiguration();
							}
							else if (iMouseX > 225) {
								Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Right]);
								iRotated = 1;
								UpdateConfiguration();
							}
						}
						else if ((iRotated == -1) && (iMouseX > 192)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Right]);
							iRotated = 0;
							UpdateConfiguration();
						}
						else if ((iRotated == 1) && (iMouseX < 225)) {
							Cube.Transform(cube.iFace, cube.tTransform[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1, (int) Axis.X, (int) Slice.Bottom, (int) Direction.Left]);
							iRotated = 0;
							UpdateConfiguration();
						}
						break;
					default:break;
				}
			}
			picCube.Refresh();
		}

		private void picCube_MouseLeave(object sender, System.EventArgs e) {
			iMouseX = iMouseY = 0;
			bMouseDown = false;
			picCube.Refresh();
		}

		private void picCube_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
			if (bMouseDown == false) {
				bMouseDown = true;
				iRotated = 0;
				if ((iMouseX > 70) && (iMouseX < 114) && (iMouseY > 116) && (iMouseY < 160))  // front face, top square
					iMouseSpot = 1;
				else if ((iMouseX > 26) && (iMouseX < 70) && (iMouseY > 160) && (iMouseY < 204))  // front face, left square
					iMouseSpot = 2;
				else if ((iMouseX > 114) && (iMouseX < 158) && (iMouseY > 160) && (iMouseY < 204))  // front face, right square
					iMouseSpot = 3;
				else if ((iMouseX > 70) && (iMouseX < 114) && (iMouseY > 204) && (iMouseY < 248))  // front face, bottom square
					iMouseSpot = 4;
				else if ((iMouseX > 70) && (iMouseX < 147) && (iMouseY > 94) && (iMouseY < 116))  // top face, front square
					iMouseSpot = 5;
				else if ((iMouseX > 136) && (iMouseX < 214) && (iMouseY > 50) && (iMouseY < 72))  // top face, back square
					iMouseSpot = 6;
				else if ((iMouseX > 59) && (iMouseX < 136) && (iMouseY > 72) && (iMouseY < 94))  // top face, left square
					iMouseSpot = 7;
				else if ((iMouseX > 147) && (iMouseX < 224) && (iMouseY > 72) && (iMouseY < 94))  // top face, right square
					iMouseSpot = 8;
				else if ((iMouseX > 192) && (iMouseX < 225) && (iMouseY > 72) && (iMouseY < 138))  // right face, top square
					iMouseSpot = 9;
				else if ((iMouseX > 158) && (iMouseX < 192) && (iMouseY > 138) && (iMouseY < 204))  // right face, left square
					iMouseSpot = 10;
				else if ((iMouseX > 225) && (iMouseX < 258) && (iMouseY > 94) && (iMouseY < 160))  // right face, right square
					iMouseSpot = 11;
				else if ((iMouseX > 192) && (iMouseX < 225) && (iMouseY > 160) && (iMouseY < 226))  // right face, bottom square
					iMouseSpot = 12;
				picCube.Refresh();
			}
		}

		private void picCube_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
			bMouseDown = false;
			iMouseSpot = 0;
			picCube.Refresh();
		}

		private void btnSolve_Click(object sender, System.EventArgs e) {     // start the solution-finding thread
			if (cube.thread == null) {     // no running thread - start one, turn off buttons, etc.
				lstSolution.Items.Clear();
				EnableButtons(false);
				btnSolve.Text = "Abort";
				cube.thread = new Thread(new ThreadStart(cube.Solve));
				cube.thread.Priority = ThreadPriority.BelowNormal;
				cube.thread.Start();
			}
			else {     // thread is already running - tell it to abort and turn the buttons on
				EnableButtons(true);
				btnSolve.Text = "Find Solution";
				cube.bStop = true;
			}
		}

		#endregion

		#region Other event handlers

		private void UpdateConfiguration() {
			// This function draws all of the colors in the Configuration box in the upper-right-hand corner of the window.
			txtFace11.BackColor = colors[cube.iFace[0, 0] - 1];
			txtFace12.BackColor = colors[cube.iFace[0, 1] - 1];
			txtFace13.BackColor = colors[cube.iFace[0, 2] - 1];
			txtFace14.BackColor = colors[cube.iFace[0, 3] - 1];
			txtFace15.BackColor = colors[cube.iFace[0, 4] - 1];
			txtFace16.BackColor = colors[cube.iFace[0, 5] - 1];
			txtFace17.BackColor = colors[cube.iFace[0, 6] - 1];
			txtFace18.BackColor = colors[cube.iFace[0, 7] - 1];
			txtFace19.BackColor = colors[cube.iFace[0, 8] - 1];
			txtFace21.BackColor = colors[cube.iFace[1, 0] - 1];
			txtFace22.BackColor = colors[cube.iFace[1, 1] - 1];
			txtFace23.BackColor = colors[cube.iFace[1, 2] - 1];
			txtFace24.BackColor = colors[cube.iFace[1, 3] - 1];
			txtFace25.BackColor = colors[cube.iFace[1, 4] - 1];
			txtFace26.BackColor = colors[cube.iFace[1, 5] - 1];
			txtFace27.BackColor = colors[cube.iFace[1, 6] - 1];
			txtFace28.BackColor = colors[cube.iFace[1, 7] - 1];
			txtFace29.BackColor = colors[cube.iFace[1, 8] - 1];
			txtFace31.BackColor = colors[cube.iFace[2, 0] - 1];
			txtFace32.BackColor = colors[cube.iFace[2, 1] - 1];
			txtFace33.BackColor = colors[cube.iFace[2, 2] - 1];
			txtFace34.BackColor = colors[cube.iFace[2, 3] - 1];
			txtFace35.BackColor = colors[cube.iFace[2, 4] - 1];
			txtFace36.BackColor = colors[cube.iFace[2, 5] - 1];
			txtFace37.BackColor = colors[cube.iFace[2, 6] - 1];
			txtFace38.BackColor = colors[cube.iFace[2, 7] - 1];
			txtFace39.BackColor = colors[cube.iFace[2, 8] - 1];
			txtFace41.BackColor = colors[cube.iFace[3, 0] - 1];
			txtFace42.BackColor = colors[cube.iFace[3, 1] - 1];
			txtFace43.BackColor = colors[cube.iFace[3, 2] - 1];
			txtFace44.BackColor = colors[cube.iFace[3, 3] - 1];
			txtFace45.BackColor = colors[cube.iFace[3, 4] - 1];
			txtFace46.BackColor = colors[cube.iFace[3, 5] - 1];
			txtFace47.BackColor = colors[cube.iFace[3, 6] - 1];
			txtFace48.BackColor = colors[cube.iFace[3, 7] - 1];
			txtFace49.BackColor = colors[cube.iFace[3, 8] - 1];
			txtFace51.BackColor = colors[cube.iFace[4, 0] - 1];
			txtFace52.BackColor = colors[cube.iFace[4, 1] - 1];
			txtFace53.BackColor = colors[cube.iFace[4, 2] - 1];
			txtFace54.BackColor = colors[cube.iFace[4, 3] - 1];
			txtFace55.BackColor = colors[cube.iFace[4, 4] - 1];
			txtFace56.BackColor = colors[cube.iFace[4, 5] - 1];
			txtFace57.BackColor = colors[cube.iFace[4, 6] - 1];
			txtFace58.BackColor = colors[cube.iFace[4, 7] - 1];
			txtFace59.BackColor = colors[cube.iFace[4, 8] - 1];
			txtFace61.BackColor = colors[cube.iFace[5, 0] - 1];
			txtFace62.BackColor = colors[cube.iFace[5, 1] - 1];
			txtFace63.BackColor = colors[cube.iFace[5, 2] - 1];
			txtFace64.BackColor = colors[cube.iFace[5, 3] - 1];
			txtFace65.BackColor = colors[cube.iFace[5, 4] - 1];
			txtFace66.BackColor = colors[cube.iFace[5, 5] - 1];
			txtFace67.BackColor = colors[cube.iFace[5, 6] - 1];
			txtFace68.BackColor = colors[cube.iFace[5, 7] - 1];
			txtFace69.BackColor = colors[cube.iFace[5, 8] - 1];
		}

		private void picCube_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			// prepare drawing tools
			System.Drawing.Graphics g = e.Graphics;
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			g.FillRectangle(Brushes.BlanchedAlmond, e.ClipRectangle);
			Brush[] brush = new Brush[6];
			for (int i = 0; i < 6; i++)
				brush[i] = new SolidBrush(colors[i]);
			// get drawing data
			int[] iFrontFace = cube.ComputeFaceValues(cube.iFrontFaceView, cube.iFrontFaceRotation);
			int iTopFaceView = cube.iTopFaceView[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1];
			int iTopFaceRotation = cube.iTopFaceRotation[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1];
			int[] iTopFace = cube.ComputeFaceValues(iTopFaceView, iTopFaceRotation);
			int iRightFaceView = cube.iRightFaceView[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1];
			int iRightFaceRotation = cube.iRightFaceRotation[cube.iFrontFaceView - 1, cube.iFrontFaceRotation - 1];
			int[] iRightFace = cube.ComputeFaceValues(iRightFaceView, iRightFaceRotation);
			// draw squares on front face
			for (int x = 0; x < 3; x++) {
				for (int y = 0; y < 3; y++) {
					g.FillRectangle(brush[iFrontFace[y * 3 + x] - 1], 25 + 44 * x, 117 + 44 * y, 44, 44);
				}
			}
			// draw squares on top face
			for (int x = 0; x < 3; x++) {
				for (int y = 0; y < 3; y++) {
					int left = 60 + (33 * (2 - y)) + 44 * x;
					int top = 50 + 22 * y;
					g.FillPolygon(brush[iTopFace[y * 3 + x] - 1], new Point[] { new Point(left, top), new Point(left + 44, top), new Point(left + 11, top + 22), new Point(left - 33, top + 22) });
				}
			}
			// draw squares on right face
			for (int x = 0; x < 3; x++) {
				for (int y = 0; y < 3; y++) {
					int left = 158 + 33 * x;
					int top = 73 + (22 * (2 - x)) + 44 * y;
					g.FillPolygon(brush[iRightFace[y * 3 + x] - 1], new Point[] { new Point(left, top), new Point(left, top + 44), new Point(left + 33, top + 22), new Point(left + 33, top - 22) });
				}
			}
			// draw lines that outline the cube
			// front face
			Pen pen = new Pen(Brushes.Black, 2);
			g.DrawLine(pen, 26, 116, 26, 248);
			g.DrawLine(pen, 70, 116, 70, 248);
			g.DrawLine(pen, 114, 116, 114, 248);
			g.DrawLine(pen, 158, 116, 158, 248);
			g.DrawLine(pen, 26, 116, 158, 116);
			g.DrawLine(pen, 26, 160, 158, 160);
			g.DrawLine(pen, 26, 204, 158, 204);
			g.DrawLine(pen, 26, 248, 158, 248);
			// top face
			g.DrawLine(pen, 26, 116, 126, 50);
			g.DrawLine(pen, 70, 116, 170, 50);
			g.DrawLine(pen, 114, 116, 214, 50);
			g.DrawLine(pen, 158, 116, 258, 50);
			g.DrawLine(pen, 126, 50, 258, 50);
			g.DrawLine(pen, 93, 72, 225, 72);
			g.DrawLine(pen, 60, 94, 192, 94);
			// right face
			g.DrawLine(pen, 258, 50, 258, 182);
			g.DrawLine(pen, 225, 72, 225, 204);
			g.DrawLine(pen, 192, 94, 192, 226); 
			g.DrawLine(pen, 158, 160, 258, 94);
			g.DrawLine(pen, 158, 204, 258, 138);
			g.DrawLine(pen, 158, 248, 258, 182);
			// highlight the square that the user is hovering over or manipulating
			if ((iMouseX > 0) || (iMouseY > 0)) {
				if ((bMouseDown == true) && (iMouseSpot != 0)) {   // highllght the spot
					Pen penHighlight = new Pen(Brushes.Cyan, 1);
					if (iMouseSpot == 1)
						g.DrawRectangle(penHighlight, 70, 116, 44, 44);
					else if (iMouseSpot == 2)
						g.DrawRectangle(penHighlight, 26, 160, 44, 44);
					else if (iMouseSpot == 3)
						g.DrawRectangle(penHighlight, 114, 160, 44, 44);
					else if (iMouseSpot == 4)
						g.DrawRectangle(penHighlight, 70, 204, 44, 44);
					else if (iMouseSpot == 5)
						g.DrawPolygon(penHighlight, new Point[] { new Point(70, 116), new Point(103, 94), new Point(147, 94), new Point(114, 116) } );
					else if (iMouseSpot == 6)
						g.DrawPolygon(penHighlight, new Point[] { new Point(136, 72), new Point(170, 50), new Point(214, 50), new Point(180, 72) } );
					else if (iMouseSpot == 7)
						g.DrawPolygon(penHighlight, new Point[] { new Point(59, 94), new Point(92, 72), new Point(136, 72), new Point(103, 94) } );
					else if (iMouseSpot == 8)
						g.DrawPolygon(penHighlight, new Point[] { new Point(147, 94), new Point(180, 72), new Point(224, 72), new Point(191, 94) } );
					else if (iMouseSpot == 9)
						g.DrawPolygon(penHighlight, new Point[] { new Point(192, 94), new Point(225, 72), new Point(225, 116), new Point(192, 138) } );
					else if (iMouseSpot == 10)
						g.DrawPolygon(penHighlight, new Point[] { new Point(158, 160), new Point(192, 138), new Point(192, 182), new Point(158, 204) } );
					else if (iMouseSpot == 11)
						g.DrawPolygon(penHighlight, new Point[] { new Point(225, 116), new Point(258, 94), new Point(258, 138), new Point(225, 160) } );
					else if (iMouseSpot == 12)
						g.DrawPolygon(penHighlight, new Point[] { new Point(192, 182), new Point(225, 160), new Point(225, 204), new Point(192, 226) } );
				}
				else {  // highlight if hovering over the square
					Pen penHighlight = new Pen(Brushes.White, 2);
					if ((iMouseX > 70) && (iMouseX < 114) && (iMouseY > 116) && (iMouseY < 160))  // front face, top square
						g.DrawRectangle(penHighlight, 70, 116, 44, 44);
					else if ((iMouseX > 26) && (iMouseX < 70) && (iMouseY > 160) && (iMouseY < 204))  // front face, left square
						g.DrawRectangle(penHighlight, 26, 160, 44, 44);
					else if ((iMouseX > 114) && (iMouseX < 158) && (iMouseY > 160) && (iMouseY < 204))  // front face, right square
						g.DrawRectangle(penHighlight, 114, 160, 44, 44);
					else if ((iMouseX > 70) && (iMouseX < 114) && (iMouseY > 204) && (iMouseY < 248))  // front face, bottom square
						g.DrawRectangle(penHighlight, 70, 204, 44, 44);
					else if ((iMouseX > 70) && (iMouseX < 147) && (iMouseY > 94) && (iMouseY < 116))  // top face, front square
						g.DrawPolygon(penHighlight, new Point[] { new Point(70, 116), new Point(103, 94), new Point(147, 94), new Point(114, 116) } );
					else if ((iMouseX > 136) && (iMouseX < 214) && (iMouseY > 50) && (iMouseY < 72))  // top face, back square
						g.DrawPolygon(penHighlight, new Point[] { new Point(136, 72), new Point(170, 50), new Point(214, 50), new Point(180, 72) } );
					else if ((iMouseX > 59) && (iMouseX < 136) && (iMouseY > 72) && (iMouseY < 94))  // top face, left square
						g.DrawPolygon(penHighlight, new Point[] { new Point(59, 94), new Point(92, 72), new Point(136, 72), new Point(103, 94) } );
					else if ((iMouseX > 147) && (iMouseX < 224) && (iMouseY > 72) && (iMouseY < 94))  // top face, right square
						g.DrawPolygon(penHighlight, new Point[] { new Point(147, 94), new Point(180, 72), new Point(224, 72), new Point(191, 94) } );
					else if ((iMouseX > 192) && (iMouseX < 225) && (iMouseY > 72) && (iMouseY < 138))  // right face, top square
						g.DrawPolygon(penHighlight, new Point[] { new Point(192, 94), new Point(225, 72), new Point(225, 116), new Point(192, 138) } );
					else if ((iMouseX > 158) && (iMouseX < 192) && (iMouseY > 138) && (iMouseY < 204))  // right face, left square
						g.DrawPolygon(penHighlight, new Point[] { new Point(158, 160), new Point(192, 138), new Point(192, 182), new Point(158, 204) } );
					else if ((iMouseX > 225) && (iMouseX < 258) && (iMouseY > 94) && (iMouseY < 160))  // right face, right square
						g.DrawPolygon(penHighlight, new Point[] { new Point(225, 116), new Point(258, 94), new Point(258, 138), new Point(225, 160) } );
					else if ((iMouseX > 192) && (iMouseX < 225) && (iMouseY > 160) && (iMouseY < 226))  // right face, bottom square
						g.DrawPolygon(penHighlight, new Point[] { new Point(192, 182), new Point(225, 160), new Point(225, 204), new Point(192, 226) } );
				}
			}
		}

		private void timerUpdate_Tick(object sender, System.EventArgs e) {     // periodically update the status line, and monitor for completion of solution-finding thread
			if (cube.thread != null) {
				txtStatus.Text = cube.strStatus;
				if (cube.bDone == true) {
					EnableButtons(true);
					btnSolve.Text = "Find Solution";
					cube.thread = null;
					lstSolution.BeginUpdate();
					lstSolution.Items.Clear();
					SolutionStep s = cube.solution;
					while (s != null) {
						string strTransform;
						if (s.transform == null)
							strTransform = "Start";
						else if (s.transform.axis == Axis.X) {
							strTransform = (s.transform.slice == Slice.Top ? "X Top " : "X Bottom ");
							strTransform += (s.transform.direction == Direction.Left ? "Left" : "Right");
						}
						else if (s.transform.axis == Axis.Y) {
							strTransform = (s.transform.slice == Slice.Left ? "Y Left " : "Y Right ");
							strTransform += (s.transform.direction == Direction.Up ? "Up" : "Down");
						}
						else {
							strTransform = (s.transform.slice == Slice.Front ? "Z Front " : "Z Back ");
							strTransform += (s.transform.direction == Direction.CCW ? "CCW" : "CW");
						}
						lstSolution.Items.Insert(0, new ListViewItem(new string[] { s.iStepNumber.ToString(), strTransform, s.iScore.ToString() } ) );
						s = s.parent;
					}
					lstSolution.EndUpdate();
				}
			}
		}

		private void CubeSearch_Closing(object sender, System.ComponentModel.CancelEventArgs e) {     // if application is closing, abort any search thread
			if (cube.thread != null)
				cube.thread.Abort();
		}

		private void trackThreshold_Scroll(object sender, System.EventArgs e) {     // Threshold slider function
			cube.iThreshold = trackThreshold.Value;
		}

		#endregion

	}
}
